;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of PENGUIN CROSS 
;;  Copyright (C) 2020 Francesc Bellido Delgado
;;  Copyright (C) 2020 David Mulero Pérez
;;  Copyright (C) 2020 Wei He
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

.include "entity_man.h.s" 
.include "../cpct_functions.h.s"
.include "generator_man.h.s"
.include "template_man.h.s"
.include "game_man.h.s"
.include "system/render_sys.h.s"
.include "system/animation_sys.h.s"

;Etiquetas de entidades
entity_array:
	.ds entity_size * max_entities
nextFreeEntity:
	.db 0x00, 0x00
lastEntity:
	.db 0x00, 0x00
entityCount:
	.db 0x00

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Entity_man_init
;;Inicializacion de las entidades
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: hl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
entity_man_init:	

	ld hl, #entity_array
	ld (lastEntity), hl
	ld (nextFreeEntity), hl

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Entity_man_create
;;Creación de una nueva entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: hl dirección del template a copiar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: a si es 0 falla
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
;;Modified register: de bc a
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
entity_man_create:

	;;Comprobamos si queda hueco libre para nuevas entidades
	ld a, (entityCount)
    	ld b, #max_entities
    	cp b ;El resultado si queda un hueco libre deberá ser un número negativo
    	jr nc, maximo ;Si el resultado es cero no creamos 
    	
    	;;Copiamos el template de la entidad que hemos recibido en hl
	ld de, (nextFreeEntity)
	ld (lastEntity), de ;Actualizamos el puntero a la última entidad
    	ld bc, #entity_size
    	ldir

    	;;Actualizamos los punteros y el contador
    	ld (nextFreeEntity), de

	ld a, (entityCount)
	inc a
	ld (entityCount), a

    	ret

    maximo:
    	ld a,#0
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Entity_man_set4destroy
;;Marca una entidad para su destrucción posterior
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: ix la entidad a marcar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
entity_man_set4destroy:

	ld a, e_type(ix)
	or #type_destroy ;Añadimos el tipo destroy
	ld e_type(ix), a
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Entity_man_destroy_one
;;Destruye una entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: hl la entidad a destruir
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: hl de bc
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
entity_man_destroy_one:

	ld b, #type_enemigo
	ld a, (hl) ;El primer contenido de la entidad es su tipo
	and b
	cp b
	jr z, mandestroyincrementarfila


	ld b, #type_plataforma
	ld a, (hl) ;El primer contenido de la entidad es su tipo
	and b
	cp b
	jr nz, comprobarelementoarray

	mandestroyincrementarfila:
	  ;Si es tipo enemigo o plataforma incrementamos el número de enemigos disponibles en la fila
	  push hl
	  pop ix 
	  ld a, e_fila(ix) ;Fila a incrementar hueco
	  call generator_man_incrementarFila

	comprobarelementoarray:
	;Comprobamos si es el último elemento de array
	ld de, (lastEntity)
	ld a, h
	cp d
	jr nz, borrar ;Si h y d son diferentes borramos
	ld a, l
	cp e
	jr nz, borrar ;Si l y e son diferentes borramos

	;Es el último elemento, solo tenemos que modificar su primer byte
	ld (hl), #type_invalid ;Byte de tipo
	ld (nextFreeEntity), hl
	;Actualizamos el puntero al último elemento
	ld hl, (lastEntity)
	ld de, #-entity_size ;Ponemos la entidad en negativo para restar con el add
	add hl, de
	ld (lastEntity), hl
	jr destroy_finish

    borrar: ;La entidad a borrar no es la última
	;Reemplazamos la entidad a borrar con la última entidad existente
	ld de, (lastEntity)
	ld bc, #entity_size
	ex de, hl ;Dejamos en 'hl' la última entidad y en 'de' la entidad a borrar
	ldir

	;Modificamos el último elemento para que se convierta en espacio libre
	ld hl, (lastEntity)
	ld (hl), #type_invalid ;Byte de tipo
	ld (nextFreeEntity), hl

	;Actualizamos el puntero al último elemento
	ld hl, (lastEntity)
	ld de, #-entity_size ;Ponemos la entidad en negativo para restar con el add
	add hl, de
	ld (lastEntity), hl

    destroy_finish:
    	;Restamos uno al contador de entidades
	ld a, (entityCount)
	dec a
	ld (entityCount), a

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;entity_man_update
;;Recorre las entidades y destruye las que están marcadas
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
entity_man_update:

	ld hl, #entity_array
	ld a, (entityCount) 

	destroy_start:
	  push af
	  push hl

	  ;Comprobamos si esta marcado para destruir
	  ld a, (hl)	
	  ld b, #type_destroy
	  and b
	  cp b
	  jr nz, next_destroy ;Si no tiene el tipo destroy pasamos a la siguiente entidad

	  call entity_man_destroy_one 

	  pop hl
	  pop af
	  dec a
	  ret z ;Si hemos recorrido todas las entidades acabamos
	  jr nz, destroy_start

	next_destroy:
	  pop hl
	  pop af
	  dec a
	  ret z ;Si hemos recorrido todas las entidades acabamos

	  ld bc, #entity_size 
	  add hl, bc ;Avanzamos hl a la siguiente entidad
	  jr	destroy_start

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;entity_man_forAll
;;Recorre las entidades e invoca la función de hl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: hl la función a invocar
;	   d  signature
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
entity_man_forAll:

	ld a, (entityCount)
	ld ix, #entity_array 

    forAll_loop:
    	push af
    	push hl

    	ld a, e_comp(ix)
    	and d
    	cp d
    	push de
    	jr nz, forAll_next ;Si no tiene la componente 'd' pasamos a la siguiente entidad

    	ld bc, #forAll_next
    	push bc ;Guardamos la dirección 'bc' al final de la pila
    	jp (hl) ;Inversión de control (realizamos la rutina 'hl')
    		  ;Al realizar ret volveremos a la dirección 'bc'

    forAll_next:
      pop de
    	pop hl
    	pop af
	dec a
	ret z ;Si hemos recorrido todas las entidades acabamos

	ld bc, #entity_size
	add ix, bc ;Avanzamos a la siguiente entidad
	jr forAll_loop

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;entity_man_forAllMatching2
;;Recorre las entidades e invoca la función de hl que cumple las condiciones
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: hl la función a invocar
;	   d  comp
;	   e  tipo
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
entity_man_forAllMatching2:

	ld a, (entityCount)
	ld ix, #entity_array 

    forAllMatching_loop:
    	push af
    	push hl
    	push de

    	ld a, e_comp(ix)
    	and d
    	cp d
    	jr nz, forAllMatching_next ;Si no tiene la componente 'd' pasamos a la siguiente entidad

    	ld a, e_type(ix)
    	and e
    	cp e
    	jr nz, forAllMatching_next ;Si no tiene el tipo 'e' pasamos a la siguiente entidad

    	ld bc, #forAllMatching_next
    	push bc ;Guardamos la dirección 'bc' al final de la pila
    	jp (hl) ;Inversión de control (realizamos la rutina 'hl')
    		  ;Al realizar ret volveremos a la dirección 'bc'

    forAllMatching_next:
      pop de
    	pop hl
    	pop af
	dec a
	ret z ;Si hemos recorrido todas las entidades acabamos

	ld bc, #entity_size
	add ix, bc ;Avanzamos a la siguiente entidad
	jr forAllMatching_loop


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;entity_man_decrementarContador
;;Decrementar el contador
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: ix entidad a modificar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: a
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
entity_man_decrementarContador:

	ld a, #0x00
	cp e_contador(ix)
	jr z, escero 

		;Si el contador no es cero lo reducimos
		dec e_contador(ix)
		ret

	escero: ;Si el contador es 0 introducimos en el contador
		  ;la cantidad ide su constante
	ld a, e_constContador(ix)
	ld e_contador(ix), a

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Entity_man_get_last_entity
;;Devuelve last entity
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: ix: lastEntity
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
entity_man_get_last_entity:
	
	ld ix, (lastEntity)

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Entity_man_get_entity_array
;;Devuelve entity array pointer
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: ix: entity array pointer
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
entity_man_get_entity_array:
	
	ld ix, #entity_array

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Entity_man_forPlayerAndAll
;;Recorre el array de entidades y empareja jugador y 
;;otra entidad que cumpla las condiciones
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: 
;;	hl funcion a invocar
;;     d signature1 componente
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: a ix iy bc 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
entity_man_forPlayerAndAll:
	
	ld a, (entityCount)
	ld ix, #entity_array ;; ix jugador, ya que siempre es el primer elemento del array (la usaremos en colision_sys_check_one_colision)
	ld iy, #entity_array ;; iy array del resto de elementos

	dec a
	ret z ;Si solo hay una entidad acabamos

	ld bc, #entity_size
	add iy, bc ;Avanzamos iy para ir a la primera entidad no player

    forPlayerAll_loop:
    	push af
    	push hl

    	ld a, e_comp(iy) ;Comprobamos componente
    	and d
    	cp d
    	push de
    	jr nz, forPlayerAll_next ;Si no tiene la componente 'd' pasamos a la siguiente entidad

    	ld bc, #forPlayerAll_next
    	push bc ;Guardamos la dirección 'bc' al final de la pila
    	jp (hl) ;Inversión de control (realizamos la rutina 'hl')
    		  ;Al realizar ret volveremos a la dirección 'bc'

    forPlayerAll_next:
      pop de
    	pop hl
    	pop af
	dec a
	ret z ;Si hemos recorrido todas las entidades acabamos

	ld bc, #entity_size
	add iy, bc ;Avanzamos a la siguiente entidad en el array de no player
	jr forPlayerAll_loop

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Entity_man_resetJugador
;;resetear jugador
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: a hl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
entity_man_resetJugador:
	
	ld ix, #entity_array
	ld a, #0x20			;X
	ld e_x(ix), a
	ld a, #0xE0			;Y
	ld e_y(ix), a
	ld a, #0x00			;VX,VY, FILA
	ld e_vx(ix), a		
	ld e_vy(ix), a	   
	ld e_fila(ix), a	
	call game_man_setGoAnimationDead 
	ld a, #0x02
	ld e_contador(ix), a	

	ld hl, #_spr_penguin_0	;SPRITE
	ld e_sprite(ix), l	
	ld e_sprite+1(ix), h	

	ld a, #player_firstmoveY
	call game_man_setDireccionYPlayer
	ld a, #player_firstmoveX
	call game_man_setDireccionXPlayer	

	call game_man_resetDirection
	;;Hace flip a los sprites que toque 
	call anim_sys_reset

	ret		

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Entity_man_reset
;;Resetea todas las variables
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: a bc hl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
entity_man_reset:
  	
	ld hl, #entity_array
	ld a, (entityCount) 

	reset_destroy_start:
	  push af
	  push hl

	  call entity_man_destroy_one 

	  pop hl
	  pop af
	  dec a
	  jr z, entity_man_reset_sig ;Si hemos recorrido todas las entidades podemos resetear los valores


	  ld bc, #entity_size 
	  add hl, bc ;Avanzamos hl a la siguiente entidad
	  jr	reset_destroy_start

	entity_man_reset_sig:
	  ld a,#0
	  ld (entityCount),a

	  ld hl, #entity_array
  	  ld (lastEntity), hl
	  ld (nextFreeEntity), hl

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;entity_man_getPlayer
;;devuelve el jugador 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: ix jugador
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: hl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
entity_man_getPlayer:	

	ld ix, #entity_array

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;entity_man_reset_huron
;;Reseteamos el sprite de huron
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
entity_man_reset_huron:
	
	ld a, (entityCount)
	ld ix, #entity_array ;;entidad

	dec a
	ret z ;Si solo hay una entidad acabamos

	ld bc, #entity_size
	add ix, bc ;Avanzamos iy para ir a la primera entidad no player

    huronreset_loop:
    	push af

    	ld a, e_fila(ix)
    	cp #6
    	jr nz, huronreset_next ;Si no está en la fila 6 pasamos a la siguiente entidad

    	ld a, e_vx(ix)
	cp #0x80
	jr nc, huronnoflip

	;;Giramos primer sprite
   	ld hl, #_spr_huron_0
  	ld  b, #16
   	ld  c, #11
	call cpct_hflipSpriteM0_asm

	;;Giramos segundo sprite
   	ld hl, #_spr_huron_1
  	ld  b, #16
   	ld  c, #11
	call cpct_hflipSpriteM0_asm

	huronnoflip:
	pop af
	ret

    huronreset_next:

    	pop af
	dec a
	ret z ;Si hemos recorrido todas las entidades acabamos

	ld bc, #entity_size
	add ix, bc ;Avanzamos a la siguiente entidad
	jr huronreset_loop

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;entity_man_resetPlayerLastPtr
;;devuelve el jugador 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: ix jugador
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: hl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
entity_man_resetPlayerLastPtr:	

	ld ix, #entity_array
	ld a, #0x00
	ld e_lastPtr(ix), a
      ld a, #0x70
      ld e_lastPtr+1(ix),a

	ret