;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of PENGUIN CROSS 
;;  Copyright (C) 2020 Francesc Bellido Delgado
;;  Copyright (C) 2020 David Mulero Pérez
;;  Copyright (C) 2020 Wei He
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

.include "../cpct_functions.h.s"
.include "cpctelera.h.s"
.include "game_man.h.s"
.include "entity_man.h.s"
.include "template_man.h.s"
.include "system/render_sys.h.s"
.include "system/physics_sys.h.s"
.include "system/input_sys.h.s"
.include "system/colision_sys.h.s"
.include "system/ia_sys.h.s"
.include "system/animation_sys.h.s"
.include "generator_man.h.s"
.include "state_man.h.s"
.include "../system/music_sys.h.s"

;;Datos del juego

vida:
	.db 0x06
vidaExtra:
	.db 0x00
direccionX_player:
	.db player_firstmoveX
direccionY_player:
	.db player_firstmoveY
lastFila_player:
	.db 0x00
metas_alcanzada:
	.db 0x00
tiempo:
	.db 0x00
nivel:
	.db 0x01

contador_tiempo:
	.db constcontadortiempo

contador_inicio:
	.db constcontadorinicio

contador_congelar:
	.db 0x00

string_inicio: 
    .asciz "  "

last_direccion_player:
	.db player_up
current_direccion_player:
	.db player_up

bool_llamar_update_time:
	.db 0x00

go_animation_dead:
  .db 0x00

go_animation_meta:
  .db 0x00

;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Game_man_init	
;;Inicializacion del juego
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: c hl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_init:

	call render_sys_init
	call entity_man_init
	call input_sys_init
	call generator_man_init
	ret 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Game_man_play
;;Bucle infito del juego	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_play:

  ld hl, #player_template
	call entity_man_create ;Creamos al jugador

	ld de,#0xC000
	call render_sys_setBufferPtr
	call game_man_crearTextoNivel
	call render_sys_fondo
	call render_sys_inicializarTiempo	

	ld de,#0x8000
	call render_sys_setBufferPtr
	call game_man_crearTextoNivel
	call render_sys_fondo
	call render_sys_inicializarTiempo

	ld a,(nivel)
	call generator_man_generatorItems

	call game_man_crearHUD
	call game_man_crearMeta

     ;Bucle juego
     game_man_loop:
	
     ld a, (contador_inicio)
     cp #0x00
     jr nz, game_man_playseguir

     call input_sys_update

     ld a, (bool_llamar_update_time)
     cp #0x00
     jr z, game_man_playseguir

     call render_update_tiempo

     ld a, #0x00
     ld (bool_llamar_update_time), a

     game_man_playseguir:
  
     call physics_sys_update
     cpctm_setBorder_asm HW_BLACK ;;Necesario por si la poción pone el borde en rojo
     call ia_sys_update
     call render_sys_update
     call game_man_completarMapa
     call colision_sys_update
     call entity_man_update

     ld a, (contador_congelar)
     cp #0
     jr nz, game_man_playsaltogenerator
     
     call generator_man_update

     game_man_playsaltogenerator:

     call cpct_waitVSYNC_asm
	   call render_sys_change_screen

	   ld a, (contador_inicio)
     cp #0x00
     jr z, game_man_escero

     call game_man_actualizarTiempoInicio
  	 jr game_man_loop

  	 game_man_escero:

     ;;Si esta a medias de la animacion de morir, tenemos que esperar a que acabe
     ld a, (go_animation_dead)
     cp #0x00
     jr nz, game_man_loop 

     ;;Si esta a medias de la animacion de meta, tenemos que esperar a que acabe
     ld a, (go_animation_meta)
     cp #0x00
     jr nz, game_man_loop

	 ;Comprobamos si le quedan vidas
	 ld a, (vida)
	 ld b, a
	 ld a, (vidaExtra)
	 add b
	 cp #0x00
	 jr z, playerdead ;Si no le queda vidas vamos a State Game Over	 
  
	 ld a, (tiempo)
	 cp #consttiempo
	 jr z, playerdead

   ld a, (metas_alcanzada)
	 cp #0x04
	 jr nz,game_man_loop

	 call game_man_nextLevel

   jp    game_man_play
  	
   playerdead:
     call cpct_akp_stop_asm
  	 ld a, #state_gameover
		 call state_man_setStatus
		 call game_man_resetGameOver

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Game_man_getVida	
;;Devuelve vida restante del jugador
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: a vida
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: c hl a
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_getVida:

	ld a, (vida)
	ld b, a
	ld a, (vidaExtra)
	add b

	ret 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Game_man_getNivel	
;;Get nivel
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: a nivel
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: a
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_getNivel:

	ld a, (nivel)

	ret 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Game_man_getDireccionPlayer	
;;Devuelve la direccion del jugador
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: a direccion
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: a
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_getDireccionXPlayer:

	ld a, (direccionX_player)
	ret 

game_man_getDireccionYPlayer:

	ld a, (direccionY_player)
	ret 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Game_man_setDireccionPlayer	
;;Modifica la direccion del jugador
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: a (nueva dirección)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_setDireccionXPlayer:

	ld (direccionX_player), a
	ret 

game_man_setDireccionYPlayer:

	ld (direccionY_player), a
	ret 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Game_man_decVida	
;;Decrementa vida restante del jugador
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: ix player
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: c hl a
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_decVida:

	ld a, (vidaExtra)
	cp #0
	jr z, decrementarVida
	dec a	
	ld (vidaExtra), a
	call game_man_updateVida
	ret

   decrementarVida:
	;Decrementamos en número de vidas
	ld a, (vida)
	dec a	
	ld (vida), a
	call game_man_updateVida
	
	ret 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Game_man_crearMeta
;;Crea las entidades del HUD
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: a hl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_crearMeta:

	 ;Creamos primera meta con el contenido inicial del template
   ld hl, #meta_template
	 call entity_man_create

	 ;Creamos segunda meta
   ld hl, #meta_template
	 call entity_man_create
	 call entity_man_get_last_entity
	 ld a, #0x18
	 ld e_x(ix), a

	 ;Creamos tercera meta
   ld hl, #meta_template
	 call entity_man_create
	 call entity_man_get_last_entity
	 ld a, #0x20
	 ld e_x(ix), a

	 ;Creamos cuarta meta
   ld hl, #meta_template
	 call entity_man_create
	 call entity_man_get_last_entity
	 ld a, #0x30
	 ld e_x(ix), a

	ret 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Game_man_crearHUD	
;;Crea las entidades del HUD
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: a hl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_crearHUD:

    ld a, (vida)
    ld c, a
    ld a, #0x3C

  bucle_vida:
    push bc
    push af
   
    ld hl, #_spr_vida
    ld de,#0xC000
    call render_sys_drawHud
    pop af
    push af
    ld hl, #_spr_vida
    ld de,#0x8000
    call render_sys_drawHud
    
    pop af
    pop bc

    sub a, #4
    dec c
    jr nz, bucle_vida

    ld b, a
    ld a, (vidaExtra)
    cp #0
    ret z
    ld c, a
    ld a, b

  bucleUpdate_vidaExtra1:
    push bc
    push af
    ld hl, #_spr_vidaExtra
    ld de, #0xC000
    call render_sys_drawHud
    pop af
    push af
    ld hl, #_spr_vidaExtra
    ld de, #0x8000
    call render_sys_drawHud
    pop af
    pop bc

    sub a, #4
    dec c
    jr nz, bucleUpdate_vidaExtra1
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Game_man_reset
;;Bucle infito del juego	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_reset: 

	ld a, #0x00
	ld (metas_alcanzada), a
	ld (contador_congelar),a
	ld (tiempo), a
	ld (bool_llamar_update_time), a

	ld a, #constcontadortiempo
	ld (contador_tiempo), a

	ld a, #constcontadorinicio
	ld (contador_inicio),a

	call game_man_reset_huron
	call entity_man_reset
	call generator_man_reset

	ret 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Game_man_setLastFila
;;modifica la fila del jugador	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: a last fila
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_setLastFila:

	ld (lastFila_player), a

	ret 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Game_man_getLastFila
;;devuelve la fila del jugador	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: a last fila
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_getLastFila:

	ld a, (lastFila_player)

	ret 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Game_man_incMeta
;;incrementa meta	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: a
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_incMeta:

  ld a, (metas_alcanzada)
  inc a
  ld (metas_alcanzada), a

  cp #4
  jr nz,aunFalta
  ld a, #2
  call music_sys_reproducirEfecto
  ret 

  aunFalta:
  ld a, #1
  call music_sys_reproducirEfecto
  ret 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Game_man_completarMapa
;;incrementa meta
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: a
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_completarMapa:

    call entity_man_getPlayer
    ld a, e_fila(ix)
    cp #0x00
    ret nz

    ld a, (lastFila_player)
    cp #0x06
    ret nz	

    ld a, #0x00
    ld (lastFila_player), a

    ;;Solo hará un reset de LastPtr en la última animación de muerte
    call game_man_getGoAnimationDead
    cp #0x00
    jr z, jumpreset

    call entity_man_resetPlayerLastPtr

    jumpreset:
    call render_sys_change_screen
    call render_sys_borrado_player
    call render_sys_player
    call render_sys_completarMar

    call render_sys_change_screen
    call render_sys_borrado_player
    call render_sys_player
    call render_sys_completarMar

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;game_man_update_time	
;;Decrementa vida restante del jugador
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: c hl a
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_update_time:

	ld a, (contador_inicio)
  cp #0
  ret nz

	;Decrementamos el contador congelar
	ld a, (contador_congelar)
	cp #0
	jr z,game_man_decrementartiempo
	dec a
	ld (contador_congelar),a

	game_man_decrementartiempo:
	;Decrementamos el tiempo
	ld a, (contador_tiempo)
	dec a	
	ld (contador_tiempo), a
	ret nz

	;Aumentamos tiempo
	ld a, (tiempo)
	inc a
	ld (tiempo), a

	ld a, #constcontadortiempo
	ld (contador_tiempo), a	

	ld (bool_llamar_update_time), a

	ret 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Game_man_getTime
;;Devuelve la variable tiempo
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: c
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: ac
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_getTime:
	ld a, (tiempo)
	ld c, a
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Game_man_nextLevel	
;;Cambiamos a otro nivel
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_nextLevel:

	call entity_man_resetJugador
	ld a, #0x00
	ld (lastFila_player), a

  call game_man_reset
  call render_sys_limpiar

  ld a, (nivel)
  inc a
  ld (nivel),a

  call generator_man_nextLevel

	ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Game_man_resetGameOver	
;;Volvemos a nivel 1
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_resetGameOver:

  cpctm_setBorder_asm HW_BLACK ;;Necesario por si la poción pone el borde en rojo
	
  ld a, #1
	ld (nivel),a

	call render_sys_resetTextoNivel

	ld a, #0x06
	ld (vida), a

	ld a, #0x00
	ld (vidaExtra), a

	;Necesitamos hacer reset para que el flip vuelva a estar bien de cara el nivel
	call entity_man_resetJugador 
  ld a, #0
  ld (lastFila_player),a

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Game_man_crearTextoNivel	
;;Renderizar texto nivel
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_crearTextoNivel:

	ld a,(nivel)
	call render_sys_textoNivel

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;game_man_updateContadorTiempoInicio	
;;Decrementamos el contador del tiempo de inicio (para interrupciones)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: abcdehl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_updateContadorTiempoInicio:

	ld a, (contador_inicio)
  cp #0x00
  ret z
	
	dec a	
	ld (contador_inicio), a
   
  ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Game_man_actualizarTiempoInicio	
;;Inicio del nivel
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: a contadorInicio
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_actualizarTiempoInicio:

    cp #0xDA
    jr c,actualizarTiempoInicioSeguir
    ld a, #3
    call music_sys_reproducirEfecto

    actualizarTiempoInicioSeguir:

    ld hl, #string_inicio

    ld (hl), #0x33

    cp #0xC4
    jr nc, game_man_actualizarTiempoInicioactualizar
    ld (hl), #0x5B

    cp #0xC0
    jr nc, game_man_actualizarTiempoInicioactualizar
    ld (hl), #0x32

    cp #0x84
    jr nc, game_man_actualizarTiempoInicioactualizar
    ld (hl), #0x5B

    cp #0x80
    jr nc, game_man_actualizarTiempoInicioactualizar
    ld (hl), #0x31

    cp #0x44
    jr nc, game_man_actualizarTiempoInicioactualizar
    ld (hl), #0x5B

    cp #0x40
    jr nc, game_man_actualizarTiempoInicioactualizar
    ld (hl), #0x47
    inc hl
    ld (hl), #0x4f

    cp #0x08
    jr nc, game_man_actualizarTiempoInicioactualizar
    dec hl
    ld (hl), #0x5B
    inc hl
    ld (hl), #0x5B

    game_man_actualizarTiempoInicioactualizar:

    game_man_actualizarTiempoInicionocero:
    ld c, #0x1D ;Coordenada X
    ld b, #0x80 ;Coordenada Y
    call render_sys_getBufferPtr
    call render_sys_getScreenPointer
    ld   iy, #string_inicio ;Puntero a la string
    call myDrawStringM0  ;Dibuja la string

    ld c, #0x1D ;Coordenada X
    ld b, #0x80 ;Coordenada Y
    call render_sys_getBufferScreenPtr
    call render_sys_getScreenPointer
    ld   iy, #string_inicio ;Puntero a la string
    call myDrawStringM0  ;Dibuja la string

    ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Game_man_reset_huron
;;Resetea sprite y ia de huron
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_reset_huron:

	call entity_man_reset_huron
	call ia_sys_reset_huron

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Game_man_updateVida	
;;Crea las entidades del HUD
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: a hl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_updateVida:

    ld a, #0xC3
    ld c, #0x20
    ld b, #0x08
    ld hl, #0xC000
    ld de, #0x0020
    add hl,de
    ex de,hl
    call render_sys_drawSolidBox

    ld a, #0xC3
    ld c, #0x20
    ld b, #0x08
    ld hl, #0x8000
    ld de, #0x0020
    add hl,de
    ex de,hl
    call render_sys_drawSolidBox

    ld a, (vida)
    cp #0
    jr z, checkVidaExtra
    ld c, a
    ld a, #0x3C

  bucleUpdate_vida:
    push bc
    push af
    ld hl, #_spr_vida
    ld de, #0xC000
    call render_sys_drawHud
    pop af
    push af
    ld hl, #_spr_vida
    ld de, #0x8000
    call render_sys_drawHud
    pop af
    pop bc

    sub a, #4
    dec c
    jr nz, bucleUpdate_vida
  
  checkVidaExtra:
    ld b, a
    ld a, (vidaExtra)
    cp #0
    ret z
    ld c, a
    ld a, b

  bucleUpdate_vidaExtra:
    push bc
    push af
    ld hl, #_spr_vidaExtra
    ld de, #0xC000
    call render_sys_drawHud
    pop af
    push af
    ld hl, #_spr_vidaExtra
    ld de, #0x8000
    call render_sys_drawHud
    pop af
    pop bc

    sub a, #4
    dec c
    jr nz, bucleUpdate_vidaExtra

    ret


 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Game_man_incVidaExtra
;;Crea las entidades del HUD
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: a 0 si no tiene puede más
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: a b
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_incVidaExtra:

	ld a,(vidaExtra)
	ld b,a
	ld a,(vida)
	add b
	cp #8
	jr nc, game_man_incVidaMaxima
	ld a,(vidaExtra)
	inc a
	ld (vidaExtra),a
	push af
	call game_man_updateVida
	pop af
	ret

	game_man_incVidaMaxima:
	ld a,#0
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Game_man_congelar
;;Set contador de congelar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: a b
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_congelar:

	ld a, #constcontadorcongelar
	ld (contador_congelar), a

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Game_man_getContadorCongelar
;;Get contador de congelar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: a contador
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_getContadorCongelar:

	ld a,(contador_congelar)

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Game_man_newDirection
;;Áctualiza la dirección actual y la anterior
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: a (nueva dirección)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: ab
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_newDirection:
	ld b, a
	ld a, (current_direccion_player)
	ld (last_direccion_player), a
	ld a, b
	ld (current_direccion_player), a
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Game_man_isSameDirection
;;Devuelve la direccion anterior y actual
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: a direccion actual, b direccion anterior
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: ab
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_isSameDirection:
	
   	ld a, (last_direccion_player)
   	ld b, a
   	ld a, (current_direccion_player)
  	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Game_man_resetDirection
;;Reseteamos la dirección del personaje
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: a
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_resetDirection:

    ld a, #player_up
    ld (last_direccion_player), a
    ld (current_direccion_player), a
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;game_man_getCurrentDirection
;;Devuelve la direccion actual
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: a 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: a
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_getCurrentDirection:	

   	ld a, (current_direccion_player)
  	ret

 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;game_man_setCurrentDirection
;;Cambia la direccion actual
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: a
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_setCurrentDirection:	

   	ld (current_direccion_player), a
  	ret

 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;game_man_setGoAnimationDead
;;Cambia el indicador de que el personaje va a realizar la animación de muerte
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: a
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_setGoAnimationDead:

    ld (go_animation_dead), a
    ret

 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;game_man_getGoAnimationDead
;;Devuelve el indicador de que el personaje va a realizar la animación de muerte
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: a
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_getGoAnimationDead:

    ld a, (go_animation_dead)
    ret

 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;game_man_escape
;;Volvemos al menu desde game
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: a
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_escape:

    ld a, #0x00
    ld (vida), a
    ld (vidaExtra), a
    ret

 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;game_man_setGoAnimationMeta
;;Cambia el indicador de que el personaje va a realizar la animación de meta
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: a
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_setGoAnimationMeta:

    ld (go_animation_meta), a
    ret

 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;game_man_getGoAnimationMeta
;;Devuelve el indicador de que el personaje va a realizar la animación de meta
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: a
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_man_getGoAnimationMeta:

    ld a, (go_animation_meta)
    ret