;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of PENGUIN CROSS 
;;  Copyright (C) 2020 Francesc Bellido Delgado
;;  Copyright (C) 2020 David Mulero Pérez
;;  Copyright (C) 2020 Wei He
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

.include "../cpct_functions.h.s"
.include "../system/render_sys.h.s"
.include "state_man.h.s"
.include "gameover_man.h.s"
.include "template_man.h.s"
.include "system/render_sys.h.s"
.include "system/input_sys.h.s"

string: 
    .asciz "GAME OVER"

contador_sprite_gameover:
    .db #0x00

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;gameover_man_init	
;;Inicializacion del Game Over
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: c hl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
gameover_man_init:

	ret 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;menu_man_play
;;Rutina principal del Game Over	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

gameover_man_play:

	call gameover_man_render
	call cpct_waitVSYNC_asm

      ld de, #0xC508
      ld a, #41

    	game_over_man_loop:

    		push af
    		push de
    		call input_sys_gameover
    		;Si el state ha variado volvemos al bucle de state_man
        call state_man_getStatus
        cp #state_gameover
        jr nz, salirconescape
        pop de
        pop af

    		push af
    		push de
    		call gameover_man_render_player
    		pop de
    		pop af

    		ld b, #15
    		mashalts:
    		halt
    		dec b
    		jr nz, mashalts

    		inc de

    		push af
    		push de
    		call game_over_borrado_player
    		call cpct_waitVSYNC_asm
    		pop de
    		pop af

    		dec a
    		cp #0x00
    		jr nz, game_over_man_loop

    		ld a, #0x00
    		ld (contador_sprite_gameover), a
    		ld de, #0xEACF

    		push af
    		push de
    		call game_over_borrado_player
    		pop de
    		pop af

    		call gameover_man_render_muerte

    		ld b, #120
    		mashalts1:
    		halt
    		dec b
    		jr nz, mashalts1

    		jr salir    		

    	jr game_over_man_loop

    	salirconescape:
    	pop de
    	pop af

  salir:
	ld a, #state_menu
	call state_man_setStatus

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;gameover_man_render
;;Render inicial Game Over
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
gameover_man_render:   	

   	ld hl, #_spr_gameover
    ld de, #0xC250
    ld  c, #Width29
    ld  b, #Height36
   	call render_sys_drawSprite	

   	ld hl, #_spr_portal
    ld de, #0xEACF
    ld  c, #Width8
    ld  b, #Height16
   	call render_sys_drawSpriteMask	

   	ld hl, #_spr_portal
    ld de, #0xC530
    ld  c, #Width8
    ld  b, #Height16
   	call render_sys_drawSpriteMask

   	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;gameover_man_render_player
;;Renderizar el player de la animación
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: de (posicion)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
gameover_man_render_player:

	ld a, (contador_sprite_gameover)
	cp #0x00
	jr z, sprite1
	cp #0x01
	jr z, sprite2
	cp #0x02
	jr z, sprite1
	
	ld hl, #_spr_penguin_5
	jr game_over_dibujar

	sprite2:
	ld hl, #_spr_penguin_3
	jr game_over_dibujar

	sprite1:
	ld hl, #_spr_penguin_1
	jr game_over_dibujar

	game_over_dibujar:

	cp #0x03
	jr nz, incContador

	ld a, #0x00
	ld (contador_sprite_gameover), a
	jr dibujarfinal

	incContador:
	inc a
	ld (contador_sprite_gameover), a

	dibujarfinal:
    ld  c, #Width8
    ld  b, #Height16
   	call render_sys_drawSpriteMask

  ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;game_over_borrado_player
;;Borramos al player
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: de (posicion)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_over_borrado_player:

  	ld a, #0xC3
   	ld c, #Width8
   	ld b, #Height16
   	call render_sys_drawSolidBox 

   	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;gameover_man_render_muerte
;;Ponemos el sprite de muerto al player
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: de (posicion)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
gameover_man_render_muerte:
		
	ld hl, #_spr_dead_3
  ld  c, #Width8
  ld  b, #Height16
  call render_sys_drawSpriteMask

  ret