;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of PENGUIN CROSS 
;;  Copyright (C) 2020 Francesc Bellido Delgado
;;  Copyright (C) 2020 David Mulero Pérez
;;  Copyright (C) 2020 Wei He
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

.include "generator_man.h.s"
.include "../cpct_functions.h.s"
.include "cpctelera.h.s"
.include "entity_man.h.s"
.include "template_man.h.s"
.include "system/render_sys.h.s"
.include "system/ia_sys.h.s"
.include "manager/game_man.h.s"

;Hueco para enemigos disponible en cada fila
contadorFila:
	.db 0x00, 0x02, 0x02, 0x00, 0x02, 0x00, 0x04, 0x03, 0x04, 0x04, 0x04, 0x03
;Frames de espera entre creación de enemigos
contadorFrame:
	.db 0x5E, 0x87, 0x65, 0x52, 0x99, 0x00, 0x29, 0x25, 0x32, 0x0A, 0x14, 0x49
		
;Creación de personajes 
datosPersonaje1:
	;    x     y    vx    constcon
	.db 0xF5, 0xD0, 0x01, 0x03   ;fila 1
datosPersonaje2:
	;    x     y    vx    constcon
	.db 0x3F, 0xC0, 0xFF, 0x03  ;fila 2
datosPersonaje3:
	;    x     y    vx    constcon
	.db 0xF5, 0xB0, 0x01, 0x03  ;fila 3
datosPersonaje4:
	;    x     y    vx    constcon
	.db 0x3F, 0xA0, 0xFF, 0x03   ;fila 4 
datosPersonaje5:
	;    x     y    vx    constcon
	.db 0xF5, 0x90, 0x01, 0x03 ;fila 5
datosPersonaje6:
	;    x     y    vx    constcon
	.db 0x3F, 0x80, 0xFF, 0x03   ;fila 6
datosPersonaje7:
	;    x     y    vx    constcon
	.db 0xF1, 0x70, 0x01, 0x04   ;fila 7    ;;;F1  plataforma  F5  madera
datosPersonaje8: 
	;    x     y    vx    constcon
	.db 0x3F, 0x60, 0xFF, 0x04   ;fila 8
datosPersonaje9:
	;    x     y    vx    constcon
	.db 0xF1, 0x50, 0x01, 0x04  ;fila 9
datosPersonaje10:
	;    x     y    vx    constcon
	.db 0x3F, 0x40, 0xFF, 0x04   ;fila 10
datosPersonaje11:
	;    x     y    vx    constcon
	.db 0xF1, 0x30, 0x01, 0x04   ;fila 11
datosPersonaje12:
	;    x     y    vx    constcon
	.db 0x3F, 0x20, 0xFF, 0x04   ;fila 12
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;generator_man_init
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
generator_man_init:	
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;generator_man_update
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
generator_man_update:

	ld b, #0xFF

  generator_man_update_loop:

  inc b
  ld a,b
  cp #12
  ret z

	ld hl, #contadorFrame
	ld d, #0
	ld e, b
	add hl,de
	dec (hl)
	jr nz, generator_man_update_loop ;Si el contador no es cero pasamos a la siguiente fila

	;;Reseteamos contador
	ld a, b
	cp #0x06
	jr nc, generator_man_update_filaalta

	ld a, #0xA0
	jr generator_man_update_next

   generator_man_update_filaalta:
	ld a, #0x68

   generator_man_update_next:
	ld (hl),a

	;;Comprobamos que hay hueco en la fila
	ld hl, #contadorFila
	add hl,de
	ld a,(hl)
	cp #0x00
	jr z, generator_man_update_loop ;Si en la fila no hay hueco pasamos a comprobar la siguiente

	;Generamos el enemigo
	ld a, b
	ld ix, #contadorFila
	call generator_man_generarUnEnemigo
	jr generator_man_update_loop

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;generator_man_generarUnEnemigo
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:    ix: Array de filas
;;		 a: Fila
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
generator_man_generarUnEnemigo:

  push af
  cp #5
  jr c,generator_man_generarUnEnemigotemplate

  cp #6
  jr c, generator_man_generarUnhurontemplate

  call game_man_getNivel
  cp #2
  jr c,generator_man_generarUnEnemigoPlataforma
  ld hl, #plataforma_sumergido_template
  jr generator_man_generarUnEnemigoseguir
 
  generator_man_generarUnEnemigoPlataforma:
  ld hl, #plataforma_template
  jr generator_man_generarUnEnemigoseguir

  generator_man_generarUnhurontemplate:
  ld hl, #huron_template
  jr generator_man_generarUnEnemigoseguir

  generator_man_generarUnEnemigotemplate:
  ld hl, #enemigo1_template

  generator_man_generarUnEnemigoseguir:
  call entity_man_create
  cp #0
  jr nz,generarUnEnemigoseg
  pop af
  ret

  generarUnEnemigoseg:
  pop af
  ld hl, #contadorFila
  ld d,#0
  ld e,a
  add hl,de

  dec (hl)  ; decrementamos 1 el contador de la fila

  call entity_man_get_last_entity

  inc a
  ld e_fila(ix), a

  dec a
  jr nz,generator_man_generarUnEnemigoseg1
  ld iy,#datosPersonaje1
  jr generator_man_generarUnEnemigofinal
  generator_man_generarUnEnemigoseg1:
  dec a
  jr nz,generator_man_generarUnEnemigoseg2
  ld iy,#datosPersonaje2
  jr generator_man_generarUnEnemigofinal
  generator_man_generarUnEnemigoseg2:
  dec a
  jr nz,generator_man_generarUnEnemigoseg3
  ld iy,#datosPersonaje3
  jr generator_man_generarUnEnemigofinal
  generator_man_generarUnEnemigoseg3:
  dec a
  jr nz,generator_man_generarUnEnemigoseg4
  ld iy,#datosPersonaje4
  jr generator_man_generarUnEnemigofinal
  generator_man_generarUnEnemigoseg4:
  dec a
  jr nz,generator_man_generarUnEnemigoseg5
  ld iy,#datosPersonaje5
  jr generator_man_generarUnEnemigofinal
  generator_man_generarUnEnemigoseg5:
  dec a
  jr nz,generator_man_generarUnEnemigoseg6
  ld iy,#datosPersonaje6
  jr generator_man_generarUnEnemigofinal
  generator_man_generarUnEnemigoseg6:
  dec a
  jr nz,generator_man_generarUnEnemigoseg7
  ld iy,#datosPersonaje7
  jr generator_man_generarUnEnemigofinal
  generator_man_generarUnEnemigoseg7:
  dec a
  jr nz,generator_man_generarUnEnemigoseg8
  ld iy,#datosPersonaje8
  jr generator_man_generarUnEnemigofinal
  generator_man_generarUnEnemigoseg8:
  dec a
  jr nz,generator_man_generarUnEnemigoseg9
  ld iy,#datosPersonaje9
  jr generator_man_generarUnEnemigofinal
  generator_man_generarUnEnemigoseg9:
  dec a
  jr nz,generator_man_generarUnEnemigoseg10
  ld iy,#datosPersonaje10
  jr generator_man_generarUnEnemigofinal
  generator_man_generarUnEnemigoseg10:
  dec a
  jr nz,generator_man_generarUnEnemigoseg11
  ld iy,#datosPersonaje11
  jr generator_man_generarUnEnemigofinal
  generator_man_generarUnEnemigoseg11:
  dec a
  jr nz,generator_man_generarUnEnemigofinal
  ld iy,#datosPersonaje12

  generator_man_generarUnEnemigofinal:

  ld a, 0(iy)
  ld e_x(ix), a
  ld a, 1(iy)
  ld e_y(ix), a
  ld a, 2(iy)
  ld e_vx(ix),a
  ld a, 3(iy)
  ld e_constContador(ix), a

  ;;le ponemos el sprite correspondiente
  ld a, e_type(ix)
  cp #type_enemigo
  jr nz, esPlataforma

  ld a, e_fila(ix)
  cp #6
  ret z

  ld a, e_constContador(ix)
  cp #3
  jr c, osoAzul

  ld a, e_vx(ix)
  cp #80
  ret c

  ld hl, #_spr_enemigo_1
  ld e_sprite(ix), l
  ld e_sprite+1(ix), h
  ret

  osoAzul:
  ld a, e_constContador(ix)
  cp #2
  jr c, osoRojo

  ld hl, #_spr_osoazul_0
  ld e_sprite(ix), l
  ld e_sprite+1(ix), h

  ld a, e_vx(ix)
  cp #80
  ret c

  ld hl, #_spr_osoazul_1
  ld e_sprite(ix), l
  ld e_sprite+1(ix), h
  ret

 osoRojo:
  ld hl, #_spr_osorojo_0
  ld e_sprite(ix), l
  ld e_sprite+1(ix), h

  ld a, e_vx(ix)
  cp #80
  ret c

  ld hl, #_spr_osorojo_1
  ld e_sprite(ix), l
  ld e_sprite+1(ix), h
  ret
 esPlataforma:
  ld a, e_constContador(ix)
  cp #4
  ret nc

  ld hl, #_spr_madera
  ld e_sprite(ix), l
  ld e_sprite+1(ix), h
  ld a, #Width12
  ld e_w(ix), a
  ret
  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;generator_man_incrementarFila
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: a: fila
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: ade
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
generator_man_incrementarFila:
	
	push hl

	dec a ;Decrementamos a ya que por ejemplo fila 1 es valor 0
	ld hl, #contadorFila
	ld d, #0x00 ;Introducimos en de el número de la fila
	ld e, a
	add hl, de
	inc (hl) ;Incrementamos el número de huecos libres en la fila

	pop hl
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;generator_man_reset
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
generator_man_reset:

	ld ix, #contadorFila
	ld fila1(ix), #0
	ld fila2(ix), #2
	ld fila3(ix), #2
	ld fila4(ix), #0
	ld fila5(ix), #2	
	ld fila6(ix), #0
	ld fila7(ix), #4
	ld fila8(ix), #3
	ld fila9(ix), #4
	ld fila10(ix), #4
	ld fila11(ix), #4
	ld fila12(ix), #3

	ld ix, #contadorFrame
	ld fila1(ix), #0x5E
	ld fila2(ix), #0x87
	ld fila3(ix), #0x65
	ld fila4(ix), #0x52
	ld fila5(ix), #0x99
	ld fila6(ix), #0x00
	ld fila7(ix), #0x29
	ld fila8(ix), #0x25
	ld fila9(ix), #0x32
	ld fila10(ix), #0x0A
	ld fila11(ix), #0x14
	ld fila12(ix), #0x49
		
	ld ix, #datosPersonaje1
	ld 0(ix), #0xF5
	ld 1(ix), #0xD0
	ld 2(ix), #0x01
	ld 3(ix), #0x03

	ld ix, #datosPersonaje2
	ld 0(ix), #0x3F
	ld 1(ix), #0xC0
	ld 2(ix), #0xFF
	ld 3(ix), #0x03

	ld ix, #datosPersonaje3
	ld 0(ix), #0xF5
	ld 1(ix), #0xB0
	ld 2(ix), #0x01
	ld 3(ix), #0x03

	ld ix, #datosPersonaje4
	ld 0(ix), #0x3F
	ld 1(ix), #0xA0
	ld 2(ix), #0xFF
	ld 3(ix), #0x03

	ld ix, #datosPersonaje5
	ld 0(ix), #0xF5
	ld 1(ix), #0x90
	ld 2(ix), #0x01
	ld 3(ix), #0x03

	ld ix, #datosPersonaje6
	ld 0(ix), #0x3F
	ld 1(ix), #0x80
	ld 2(ix), #0xFF
	ld 3(ix), #0x03

	ld ix, #datosPersonaje7
	ld 0(ix), #0xF1
	ld 1(ix), #0x70
	ld 2(ix), #0x01
	ld 3(ix), #0x04

	ld ix, #datosPersonaje8
	ld 0(ix), #0x3F
	ld 1(ix), #0x60
	ld 2(ix), #0xFF
	ld 3(ix), #0x04

	ld ix, #datosPersonaje9
	ld 0(ix), #0xF1
	ld 1(ix), #0x50
	ld 2(ix), #0x01
	ld 3(ix), #0x04

	ld ix, #datosPersonaje10
	ld 0(ix), #0x3F
	ld 1(ix), #0x40
	ld 2(ix), #0xFF
	ld 3(ix), #0x04

	ld ix, #datosPersonaje11
	ld 0(ix), #0xF1
	ld 1(ix), #0x30
	ld 2(ix), #0x01
	ld 3(ix), #0x04

	ld ix, #datosPersonaje12
	ld 0(ix), #0x3F
	ld 1(ix), #0x20
	ld 2(ix), #0xFF
	ld 3(ix), #0x04

	ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;generator_man_incrementarFila
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: a: nivel
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: ix iy  Flag: c
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
generator_man_nextLevel:

	push af
	ld hl, #contadorFrame
	ld b, #11
	nextLevelRandom:
	push hl
	call cpct_getRandom_xsp40_u8_asm
	pop hl
	ld (hl),a
	dec b
	cp #0
	jr nz, nextLevelRandom

	pop af

	cp #11
	jp nc,generatorUltimosNiveles

	ld ix, #contadorFila

	cp #02
	jp c, generatorNextNivelFinal

	ld fila4(ix), #1
  ld fila7(ix), #3

  ld ix, #datosPersonaje2
	ld (ix), #0xF5
  ld 2(ix), #0x01

  ld ix, #datosPersonaje5
  ld (ix), #0x3F
  ld 2(ix), #0xFF
  ld 3(ix), #0x03

  ld ix, #datosPersonaje8
  ld (ix), #0x3F
  ld 2(ix), #0xFF
  ld 3(ix), #0x02



  ld ix, #contadorFila
 	ld fila6(ix), #1  
 	ld fila5(ix), #1  


	cp #03
	jp c, generatorNextNivelFinal

  ld ix, #datosPersonaje2
 	    ld 3(ix), #0x02

	ld ix, #datosPersonaje3
      ld (ix), #0x3F
      ld 2(ix), #0xFF

      ld ix, #datosPersonaje4
      ld (ix), #0xF5
      ld 2(ix), #0x01

	ld ix, #datosPersonaje7
      ld (ix), #0xF1
      ld 2(ix), #0x01
     	ld 3(ix), #0x04

      ld ix, #datosPersonaje12
      ld (ix), #0xF5
      ld 2(ix), #0x01
     	ld 3(ix), #0x03

      ld ix, #contadorFila
      ld fila1(ix), #1
      ld fila6(ix), #0 
      ld fila10(ix), #3
      ld fila12(ix), #3

  cp #04
	jp c, generatorNextNivelFinal

	ld ix, #datosPersonaje12
      ld (ix), #0x3F
      ld 2(ix), #0xFF
     	ld 3(ix), #0x03

     	ld ix, #contadorFila
      ld fila6(ix), #1
      ld fila5(ix), #2 

  cp #05
	jp c, generatorNextNivelFinal

	ld ix, #datosPersonaje11
     	ld (ix), #0xF7
      ld 2(ix), #0x01
     	ld 3(ix), #0x03

  ld ix, #datosPersonaje6
     	ld 3(ix), #0x03

  ld ix, #datosPersonaje4
     	ld 3(ix), #0x01

  ld ix, #contadorFila
      ld fila4(ix), #2

  cp #6
	jp c, generatorNextNivelFinal

	ld ix, #datosPersonaje12
      ld (ix), #0xF1
      ld 2(ix), #0x01
     	ld 3(ix), #0x04

	ld ix, #datosPersonaje11
     	ld (ix), #0x3F
      ld 2(ix), #0xFF
     	ld 3(ix), #0x04

     	ld ix, #datosPersonaje10
      ld (ix), #0xF5
      ld 2(ix), #0x01
     	ld 3(ix), #0x03

	ld ix, #datosPersonaje9
     	ld (ix), #0x3F
      ld 2(ix), #0xFF
     	ld 3(ix), #0x03

  ld ix, #datosPersonaje1
     	ld 3(ix), #0x03

  ld ix, #datosPersonaje2
     	ld 3(ix), #0x02

  ld ix, #datosPersonaje3
     	ld 3(ix), #0x03

  ld ix, #datosPersonaje4
     	ld 3(ix), #0x01

  ld ix, #datosPersonaje5
     	ld 3(ix), #0x02

  ld ix, #datosPersonaje6
     	ld 3(ix), #0x03

  ld ix, #contadorFila
      ld fila11(ix), #2  
      ld fila12(ix), #2
      ld fila1(ix), #1
      ld fila2(ix), #1
      ld fila3(ix), #2
      ld fila4(ix), #1
      ld fila5(ix), #1
      ld fila6(ix), #1

  cp #7
	jp c, generatorNextNivelFinal

	ld ix, #datosPersonaje12
     	ld 3(ix), #0x04


	ld ix, #datosPersonaje11
      ld (ix), #0x3F
      ld 2(ix), #0xFF
     	ld 3(ix), #0x03

	ld ix, #datosPersonaje10
      ld (ix), #0x3F
      ld 2(ix), #0xFF
     	ld 3(ix), #0x04

	ld ix, #datosPersonaje9
     	ld (ix), #0xF5
      ld 2(ix), #0x01
     	ld 3(ix), #0x03

     	ld ix, #datosPersonaje8
      ld (ix), #0x3F
      ld 2(ix), #0xFF
     	ld 3(ix), #0x04

	ld ix, #datosPersonaje7
     	ld (ix), #0xF7
      ld 2(ix), #0x01
     	ld 3(ix), #0x03

  ld ix, #datosPersonaje1
     	ld 3(ix), #0x01

    	ld ix, #datosPersonaje3
     	ld 3(ix), #0x03

     	ld ix, #datosPersonaje4
     	ld 3(ix), #0x03


  ld ix, #contadorFila
      ld fila9(ix), #3  
      ld fila11(ix), #3  
      ld fila1(ix), #2
      ld fila3(ix), #2
      ld fila4(ix), #2

  cp #8
	jp c, generatorNextNivelFinal

	ld ix, #datosPersonaje9
     	ld (ix), #0xF1
      ld 2(ix), #0x01
     	ld 3(ix), #0x04

  ld ix, #datosPersonaje8
      ld (ix), #0x3F
      ld 2(ix), #0xFF
     	ld 3(ix), #0x03

  ld ix, #datosPersonaje4
     	ld 3(ix), #0x02

  ld ix, #contadorFila
      ld fila3(ix), #1

  cp #9
	jp c, generatorNextNivelFinal

	ld ix, #datosPersonaje12
      ld (ix), #0xF1
      ld 2(ix), #0x01
     	ld 3(ix), #0x04

  ld ix, #datosPersonaje11
      ld (ix), #0x3F
      ld 2(ix), #0xFF
     	ld 3(ix), #0x03

  ld ix, #datosPersonaje10
      ld (ix), #0xF5
      ld 2(ix), #0x01
     	ld 3(ix), #0x02

	ld ix, #datosPersonaje9
     	ld (ix), #0x3F
      ld 2(ix), #0xFF
     	ld 3(ix), #0x04

  ld ix, #datosPersonaje8
      ld (ix), #0xF5
      ld 2(ix), #0x01
     	ld 3(ix), #0x02

	ld ix, #datosPersonaje7
     	ld (ix), #0x3F
      ld 2(ix), #0xFF
     	ld 3(ix), #0x03

  ld ix, #datosPersonaje4
     	ld 3(ix), #0x02

  ld ix, #contadorFila
      ld fila3(ix), #2

  cp #10
	jr c, generatorNextNivelFinal

	ld ix, #datosPersonaje12
      ld (ix), #0xF5
      ld 2(ix), #0x01
     	ld 3(ix), #0x03

  ld ix, #datosPersonaje9
     	ld (ix), #0x3F
      ld 2(ix), #0xFF
     	ld 3(ix), #0x03

  generatorNextNivelFinal:
	ret

	generatorUltimosNiveles:

	ld ix, #contadorFila
      ld fila12(ix), #3  
      ld fila11(ix), #3  
      ld fila10(ix), #3
      ld fila9(ix),  #3
      ld fila8(ix),  #3
      ld fila7(ix),  #3  
      ld fila6(ix), #1 
      ld fila5(ix), #2
      ld fila4(ix), #2
      ld fila3(ix), #2
      ld fila2(ix), #2
      ld fila1(ix), #2

	ld ix, #datosPersonaje12
      ld (ix), #0xF5
      ld 2(ix), #0x01
     	ld 3(ix), #0x03

  ld ix, #datosPersonaje11
      ld (ix), #0x3F
      ld 2(ix), #0xFF
     	ld 3(ix), #0x03

  ld ix, #datosPersonaje10
      ld (ix), #0xF5
      ld 2(ix), #0x01
     	ld 3(ix), #0x02

	ld ix, #datosPersonaje9
     	ld (ix), #0x3F
      ld 2(ix), #0xFF
     	ld 3(ix), #0x03

  ld ix, #datosPersonaje8
      ld (ix), #0xF5
      ld 2(ix), #0x01
     	ld 3(ix), #0x02

	ld ix, #datosPersonaje7
     	ld (ix), #0x3F
      ld 2(ix), #0xFF
     	ld 3(ix), #0x03

	ld ix, #datosPersonaje5
     	ld (ix), #0x3F
      ld 2(ix), #0xFF
     	ld 3(ix), #0x03

  ld ix, #datosPersonaje4
     	ld (ix), #0xF5
      ld 2(ix), #0x01
     	ld 3(ix), #0x03

	ld ix, #datosPersonaje3
     	ld (ix), #0x3F
      ld 2(ix), #0xFF
     	ld 3(ix), #0x03

	ld ix, #datosPersonaje2
     	ld (ix), #0xF5
      ld 2(ix), #0x01
     	ld 3(ix), #0x03

	ld ix, #datosPersonaje1
     	ld (ix), #0x3F
      ld 2(ix), #0xFF
     	ld 3(ix), #0x03
	
	ld  hl,  #datosPersonaje1
	ld b, #11

	nextLevelRandomOso:
	ld a,b
	cp #6
	jr z,noLoHace
	cp #3
	jr z,noLoHace

	push hl
	call cpct_getRandom_xsp40_u8_asm	;;	;    x     y    vx    constcon
								;;.db 0xF5, 0xD0, 0x01, 0x05  ;fila 1
	and #1
	pop hl
	cp #0
	jr z, osoVelocidadPositiva
	ld (hl), #0x3F
	inc hl
	inc hl
	ld (hl), #0xFF
	inc hl
	jr osoVelocidadRandom

	osoVelocidadPositiva:
	ld (hl), #0xF5
	inc hl
	inc hl
	ld (hl), #0x01
	inc hl

	osoVelocidadRandom:
	push hl
	call cpct_getRandom_xsp40_u8_asm
	and #1
	inc a
	pop hl
	ld (hl),a

	inc hl
	dec b
	jr nz, nextLevelRandomOso
	ret 

	noLoHace:
	ld de, #4
	add hl,de
	dec b
	jr  nz,nextLevelRandomOso
	ret z

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;generator_man_generateItems
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: a: nivel
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
generator_man_generatorItems:

	cp #01
	jr nz, generatorNextItems2

	call generator_man_generarPocion
	ret

  generatorNextItems2:
	cp #02
	jr nz, generatorNextItems3

	call generator_man_generarReloj
	ret

	generatorNextItems3:
	cp #03
	jr nz, generatoItemsFinal

	call generator_man_generarPortales
	ret

  generatoItemsFinal:

  cp #4
  ret c
 	call cpct_getRandom_xsp40_u8_asm
 	cp #80
 	jr nc, generarOtro
 	call generator_man_generarPocion
 	ret
 	generarOtro:
	cp #160
 	jr nc, generarOtro2
 	call generator_man_generarReloj
 	ret
	generarOtro2:
	call generator_man_generarPortales
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;generator_man_generarPocion
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
generator_man_generarPocion:

	ld hl, #pocion_template
	call entity_man_create

	call entity_man_get_last_entity

	ld bc,#0x00FE
  call generator_man_getRandom
 	and #0x07     			; 00000111  |_!_!_!_!_!_
 	
 	add a
 	add a
 	add a   

	ld e_x(ix),a

	ld bc,#0x020B
  call generator_man_getRandom

	ld b,a

	add a
	add a
	add a
	add a

	ld e_y(ix),a

	ld a,#14
	sub a,b
	ld e_fila(ix),a

	ld c, e_x(ix)
	ld b, e_y(ix)
	call render_sys_getBufferPtr
	call render_sys_getScreenPointer
	ld e_lastPtr(ix),l
	ld e_lastPtr+1(ix),h

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;generator_man_generarReloj
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
generator_man_generarReloj:

	ld hl, #reloj_template
	call entity_man_create

	call entity_man_get_last_entity

	ld bc,#0x00FE
  call generator_man_getRandom
  and #0x07      			; 00000111
     	
	add a
	add a
  add a
 
	ld e_x(ix),a

	ld bc,#0x020B
  call generator_man_getRandom

	ld b,a
	add a
	add a
	add a
	add a

	ld e_y(ix),a

	ld a,#14
	sub a,b
	ld e_fila(ix),a

	ld c, e_x(ix)
	ld b, e_y(ix)
	call render_sys_getBufferPtr
	call render_sys_getScreenPointer
	ld e_lastPtr(ix),l
	ld e_lastPtr+1(ix),h

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;generator_man_generarPortales
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
generator_man_generarPortales:

	ld a,#2

	generator_man_generarloop:
	push af
	ld hl, #portal_template
	call entity_man_create

	call entity_man_get_last_entity

	ld bc,#0x0007
  call generator_man_getRandom

	ld b,a

	add a
	add a
	add a

	ld e_x(ix),a
	pop af
	dec a
	cp #1
	jr nz,generator_man_generar_segundo

	ld bc,#0x0207
	push ix
	pop  de
	call ia_sys_setPortal1Pointer
	jr generator_man_generar_seguir

	generator_man_generar_segundo:
	push ix
	pop  de
	call ia_sys_setPortal2Pointer
	ld bc,#0x090D

	generator_man_generar_seguir:
	push af

  call generator_man_getRandom

	ld b,a

	add a
	add a
	add a
	add a

	ld e_y(ix),a

	ld a,#14
	sub a,b
	ld e_fila(ix),a

	ld c, e_x(ix)
	ld b, e_y(ix)
	call render_sys_getBufferPtr
	call render_sys_getScreenPointer
	ld e_lastPtr(ix),l
	ld e_lastPtr+1(ix),h

	pop af
	cp #0
	jr nz,generator_man_generarloop

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;generator_man_getRandom
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: b: valor bajo
;;	   c: valor alto
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output a: valor random
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
generator_man_getRandom:

	inc c
	getRandomPrincipio:
	call cpct_getRandom_xsp40_u8_asm
	cp b
	jr c,getRandomPrincipio
	cp c
	jr nc,getRandomPrincipio

	ret
