;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of PENGUIN CROSS 
;;  Copyright (C) 2020 Francesc Bellido Delgado
;;  Copyright (C) 2020 David Mulero Pérez
;;  Copyright (C) 2020 Wei He
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

.include "menu_man.h.s"
.include "../cpct_functions.h.s"
.include "../system/render_sys.h.s"
.include "../system/input_sys.h.s"
.include "../system/music_sys.h.s"
.include "template_man.h.s"
.include "state_man.h.s"

menu_opcion:
	.db 0x00 ; 0 jugar, 1 controles, 2 creditos, 3 musica

contador_input:
  .db 0x08 ; 

sound_option:
  .db 0x00 

string_play: 
    .asciz "PLAY"

string_controls: 
    .asciz "CONTROLS"

string_penguin:
    .asciz "PENGUIN CROSS"

string_credits:
    .asciz "CREDITS"

string_soundAll:
    .asciz "SOUND]ALL"

string_soundSFX:
    .asciz "SOUND]SFX"

string_soundOFF:
    .asciz "SOUND]OFF"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;menu_man_init	
;;Inicializacion del juego
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: c hl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
menu_man_init:

	ret 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;menu_man_play
;;Bucle infito del juego	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

menu_man_play:
	;Bucle juego

	call menu_man_render_design

  menu_man_play_loop:
    ld a, (contador_input)
    cp #0
    jr z, menu_man_play_input
    dec a
    ld (contador_input),a
    jr menu_man_play_seguir
    menu_man_play_input:
	  call input_sys_menu_update

   menu_man_play_seguir:
	 
    call menu_man_render
    call cpct_waitVSYNC_asm

	  ;Si el state ha variado volvemos al bucle de state_man
	  call state_man_getStatus
	  cp #state_menu
	  ret nz

    jr  menu_man_play_loop

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;menu_man_render
;;Renderizar botones	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
menu_man_render:

	ld a, (menu_opcion)
	dec a	;1-1 = 0 = control
	jr nz, no_control

  ld hl, #_spr_penguin_1
  ld de, #0xE506
  ld  c, #Width8
  ld  b, #Height16
  call render_sys_drawSpriteMask

  ld hl, #_spr_penguin_1
  ld  b, #Height16
  ld  c, #Width8
  call cpct_hflipSpriteM0_asm

  ld hl, #_spr_penguin_1
  ld de, #0xE532
  ld  c, #Width8
  ld  b, #Height16
  call render_sys_drawSpriteMask

  ld hl, #_spr_penguin_1
  ld  b, #Height16
  ld  c, #Width8
  call cpct_hflipSpriteM0_asm

  ret

  no_control:
	dec a	;2-2 = 0 = music
	jr nz, no_music

  ld hl, #_spr_penguin_1
  ld de, #0xE5C4
  ld  c, #Width8
  ld  b, #Height16
  call render_sys_drawSpriteMask

  ld hl, #_spr_penguin_1
  ld  b, #Height16
  ld  c, #Width8
  call cpct_hflipSpriteM0_asm

  ld hl, #_spr_penguin_1
  ld de, #0xE5F4
  ld  c, #Width8
  ld  b, #Height16
  call render_sys_drawSpriteMask

  ld hl, #_spr_penguin_1
  ld  b, #Height16
  ld  c, #Width8
  call cpct_hflipSpriteM0_asm

  ret

  no_music:
  dec a ;3-3 = 0 = credits
  jr nz, no_credit

  ld hl, #_spr_penguin_1
  ld de, #0xE688
  ld  c, #Width8
  ld  b, #Height16
  call render_sys_drawSpriteMask

  ld hl, #_spr_penguin_1
  ld  b, #Height16
  ld  c, #Width8
  call cpct_hflipSpriteM0_asm

  ld hl, #_spr_penguin_1
  ld de, #0xE6B0
  ld  c, #Width8
  ld  b, #Height16
  call render_sys_drawSpriteMask

  ld hl, #_spr_penguin_1
  ld  b, #Height16
  ld  c, #Width8
  call cpct_hflipSpriteM0_asm
 
  ret

  no_credit:   ;0 jugar

   ld hl, #_spr_penguin_1
   ld de, #0xE44E
   ld  c, #Width8
   ld  b, #Height16
   call render_sys_drawSpriteMask

   ld hl, #_spr_penguin_1
   ld  b, #Height16
   ld  c, #Width8
   call cpct_hflipSpriteM0_asm
 
   ld hl, #_spr_penguin_1
   ld de, #0xE46A
   ld  c, #Width8
   ld  b, #Height16
   call render_sys_drawSpriteMask

   ld hl, #_spr_penguin_1
   ld  b, #Height16
   ld  c, #Width8
   call cpct_hflipSpriteM0_asm

   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;menu_man_decOpcion
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
menu_man_decOpcion:

  ld a, #0x08
  ld (contador_input),a

	ld a, (menu_opcion)
	cp #0x00
	ret z

  push af
  ld d, #1
  ld e, #72
  call music_sys_reproducirNota
  pop af

	dec a
	ld (menu_opcion), a

  ld de, #0xE44E
  ld a,  #0x3C
  ld c,  #8
  ld b,  #16
  call render_sys_drawSolidBox

  ld de, #0xE506
  ld a,  #0x3C
  ld c,  #8
  ld b,  #16
  call render_sys_drawSolidBox

  ld de, #0xE46A
  ld a,  #0x3C
  ld c,  #8
  ld b,  #16
  call render_sys_drawSolidBox

  ld de, #0xE532
  ld a,  #0x3C
  ld c,  #8
  ld b,  #16
  call render_sys_drawSolidBox

  ld de, #0xE688
  ld a,  #0x3C
  ld c,  #8
  ld b,  #16
  call render_sys_drawSolidBox

  ld de, #0xE6B0
  ld a,  #0x3C
  ld c,  #8
  ld b,  #16
  call render_sys_drawSolidBox

  ld de, #0xE5C4
  ld a,  #0x3C
  ld c,  #8
  ld b,  #16
  call render_sys_drawSolidBox

  ld de, #0xE5F4
  ld a,  #0x3C
  ld c,  #8
  ld b,  #16
  call render_sys_drawSolidBox

	ret 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;menu_man_incOpcion
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
menu_man_incOpcion:

  ld a, #0x08
  ld (contador_input),a

	ld a, (menu_opcion)
	cp #0x03
	ret z

  push af
  ld d,#1
  ld e, #74
  call music_sys_reproducirNota
  pop af

	inc a
	ld (menu_opcion), a

  ld de, #0xE44E
  ld a,  #0x3C
  ld c,  #8
  ld b,  #16
  call render_sys_drawSolidBox

  ld de, #0xE506
  ld a,  #0x3C
  ld c,  #8
  ld b,  #16
  call render_sys_drawSolidBox
 
  ld de, #0xE46A
  ld a,  #0x3C
  ld c,  #8
  ld b,  #16
  call render_sys_drawSolidBox

  ld de, #0xE532
  ld a,  #0x3C
  ld c,  #8
  ld b,  #16
  call render_sys_drawSolidBox

  ld de, #0xE688
  ld a,  #0x3C
  ld c,  #8
  ld b,  #16
  call render_sys_drawSolidBox

  ld de, #0xE6B0
  ld a,  #0x3C
  ld c,  #8
  ld b,  #16
  call render_sys_drawSolidBox

  ld de, #0xE5C4
  ld a,  #0x3C
  ld c,  #8
  ld b,  #16
  call render_sys_drawSolidBox

  ld de, #0xE5F4
  ld a,  #0x3C
  ld c,  #8
  ld b,  #16
  call render_sys_drawSolidBox
	
	ret 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;menu_man_pulsarOpcion
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
menu_man_pulsarOpcion:

  ld a, #0x08
  ld (contador_input),a

  ld d,#1
  ld e, #76
  call music_sys_reproducirNota

	ld a, (menu_opcion)
	cp #0x00
	jr nz, opcionControls

	ld a, #state_game
	call state_man_setStatus
	ret

  opcionControls: 
    cp #0x01
    jr nz, opcionMusic

    ld a, #state_controls
    call state_man_setStatus
    ret

  opcionMusic: 
    cp #0x02
    jr nz, opcionCredits

    ld a, (sound_option)
    inc a
    cp #3
    jr nz, opcionMusicSeguir
    ld a, #0

  opcionMusicSeguir:
    ld (sound_option),a
    push af

    ld c, #0x0E ;Coordenada X
    ld b, #0xC0 ;Coordenada Y
    ld de, #0xC000 
    call render_sys_getScreenPointer     
    ex de,hl
    ld c, #36
    ld b, #8
    ld a, #0x3C
    call render_sys_drawSolidBox;

    pop af
    cp #0
    jr nz,opcionMusicSFX
    call music_sys_reproducirMusica
    ld   iy, #string_soundAll ;Puntero a la string
    jr opcionMusicSeguir2

  opcionMusicSFX:
    cp #1
    jr nz,opcionMusicOFF
    call music_sys_pararMusica
    ld   iy, #string_soundSFX ;Puntero a la string
    jr opcionMusicSeguir2

  opcionMusicOFF:
    call music_sys_pararMusica
    call cpct_akp_stop_asm
    ld   iy, #string_soundOFF ;Puntero a la string

  opcionMusicSeguir2:
    call cpct_setDrawCharM0_asm  
    ld c, #0x0E ;Coordenada X
    ld b, #0xC0 ;Coordenada Y
    ld de, #0xC000 
    call render_sys_getScreenPointer     

    call myDrawStringM0  ;Dibuja la string
    ret

  opcionCredits:
    ld a, #state_credits
    call state_man_setStatus
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;menu_man_render_design
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

menu_man_render_design:

   ld de, #0xC000
   ld a, #0x3C
   ld c, #64
   ld b, #256
   call render_sys_drawSolidBox

   ld hl, #_spr_igloo	
   ld de, #0xC21C
   ld  c, #Width8
   ld  b, #Height16
   call render_sys_drawSprite

   ld hl, #_spr_meta_0
   ld de, #0xC20D
   ld  c, #Width8
   ld  b, #Height16
   call render_sys_drawSprite

   ld hl, #_spr_meta_0
   ld de, #0xC22B
   ld  c, #Width8
   ld  b, #Height16
   call render_sys_drawSprite

   ld hl, #_spr_hielo 
   ld de, #0xC2E4
   ld  c, #Width16
   ld  b, #Height16
   call render_sys_drawSprite

   ld hl, #_spr_hielo 
   ld de, #0xC34C
   ld  c, #Width16
   ld  b, #Height16
   call render_sys_drawSprite

   ld    h, #6 ;D = Background PEN (0)
   ld    l, #9 ;D = Foreground PEN (0)

   call cpct_setDrawCharM0_asm  
   ld c, #0x06 ;Coordenada X
   ld b, #0x20 ;Coordenada Y
   ld de, #0xC000 
   call render_sys_getScreenPointer     
   ld   iy, #string_penguin    ;Puntero a la string
   call myDrawStringM0  ;Dibuja la string

   ld c, #0x18 ;Coordenada X
   ld b, #0x90 ;Coordenada Y
   ld de, #0xC000 
   call render_sys_getScreenPointer     
   ld   iy, #string_play ;Puntero a la string
   call myDrawStringM0  ;Dibuja la string

   call cpct_setDrawCharM0_asm  
   ld c, #0x10 ;Coordenada X
   ld b, #0xA8 ;Coordenada Y
   ld de, #0xC000 
   call render_sys_getScreenPointer     
   ld   iy, #string_controls ;Puntero a la string
   call myDrawStringM0  ;Dibuja la string

   call cpct_setDrawCharM0_asm  
   ld c, #0x0E ;Coordenada X
   ld b, #0xC0 ;Coordenada Y
   ld de, #0xC000 
   call render_sys_getScreenPointer  

   ld   a, (sound_option)
   cp #0
   jr nz, render_designOpcion1
   ld   iy, #string_soundAll ;Puntero a la string
   jr render_designFinish

   render_designOpcion1:
   cp #1
   jr nz,render_designOpcion2
   ld   iy, #string_soundSFX ;Puntero a la string
   jr render_designFinish

   render_designOpcion2:
   ld   iy, #string_soundOFF ;Puntero a la string
   jr render_designFinish

   render_designFinish:
 
   call myDrawStringM0  ;Dibuja la string
   call cpct_setDrawCharM0_asm  
   ld c, #0x12 ;Coordenada X
   ld b, #0xD8 ;Coordenada Y
   ld de, #0xC000 
   call render_sys_getScreenPointer     
   ld iy, #string_credits ;Puntero a la string
   call myDrawStringM0  ;Dibuja la string

	 ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;menu_man_init 
;;Inicializacion del juego
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: c hl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
menu_man_getSoundOption:

  ld a, (sound_option)

  ret