;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of PENGUIN CROSS 
;;  Copyright (C) 2020 Francesc Bellido Delgado
;;  Copyright (C) 2020 David Mulero Pérez
;;  Copyright (C) 2020 Wei He
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

.include "state_man.h.s"
.include "../cpct_functions.h.s"
.include "cpctelera.h.s"
.include "game_man.h.s"
.include "menu_man.h.s"
.include "gameover_man.h.s"
.include "../system/render_sys.h.s"
.include "../system/music_sys.h.s"
.include "template_man.h.s"
.include "controls_man.h.s"
.include "credits_man.h.s"

;;Estado
estado:
	.db state_menu

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;state_man_init	
;;Inicializacion del juego
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: c hl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
state_man_init:
      call game_man_init
      call menu_man_init
      call music_sys_init

      ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;state_man_update	
;;Inicializacion del juego
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: c hl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
state_man_update:

	state_man_updateloop:
  ld  de, #0x0C30
  call render_sys_set_crtc
	ld  a, (estado) ;Si estado es 1 vamos al menú
	cp #state_menu
	jr nz, state_man_updatesig 

	  call menu_man_play
    call render_sys_limpiar
	  jr state_man_updateloop

	state_man_updatesig:
    cp #state_game
    jr nz, state_man_updatesig1 ;Si estado es 2 vamos al Game

      call game_man_play
      call game_man_reset
      call render_sys_limpiar 
      jr state_man_updateloop

    ;Si el estado es 3 vamos al Controls
    state_man_updatesig1:
    cp #state_controls
    jr nz, state_man_updatesig2

      call controls_man_play
      call render_sys_limpiar 
      jr state_man_updateloop

    ;Si el estado es 4 vamos al Credits
    state_man_updatesig2:
    cp #state_credits
    jr nz, state_man_updatesig3

      call credits_man_play
      call render_sys_limpiar 
      jr state_man_updateloop

    ;Si el estado no es ninguno de los anteriores al Game Over
    state_man_updatesig3:
    cp #state_gameover
    jr nz, state_man_updatesig4

      call gameover_man_play
      call render_sys_limpiar 
      jr state_man_updateloop

    state_man_updatesig4:
    jr state_man_updateloop

 	ret

 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;state_man_setStatus	
;;Inicializacion del juego
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: a status
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
state_man_setStatus:

      ld (estado), a

      ret

 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;state_man_getStatus	
;;Inicializacion del juego
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: a status
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
state_man_getStatus:
  
      ld a, (estado)

      ret