;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of PENGUIN CROSS 
;;  Copyright (C) 2020 Francesc Bellido Delgado
;;  Copyright (C) 2020 David Mulero Pérez
;;  Copyright (C) 2020 Wei He
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

.include "../cpct_functions.h.s"
.include "animation_sys.h.s"
.include "manager/template_man.h.s"
.include "manager/entity_man.h.s"
.include "system/render_sys.h.s"
.include "manager/game_man.h.s"
.include "colision_sys.h.s"

flipped_dead:
    .db 0x00

sprite_walking_in_useX:
    .db quieto_derecha

sprite_walking_in_useY:
    .db quieto_arriba

sprite_walking_in_moveX:
    .db paso_first

sprite_walking_in_moveY:
    .db paso_first

last_animX:
    .db anim_right

last_animY:
    .db anim_up

no_quieto: 
    .db #0x00

proceso_muerte:
    .db #0x00

proceso_meta:
    .db #0x00

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Animation_sys_muerte
;;Animación perdida de vida personaje
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: ix jugador
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: c hl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
animation_sys_muerte:

    call entity_man_getPlayer
    ld a, #0x03
    ld e_contador(ix), a

    ;;¿A QUE PARTE VAMOS?
    ld a, (proceso_muerte)
    cp #0x00
    jr z, primeraparte
    cp #0x01
    jr z, segundaparte
    cp #0x02
    jr z, terceraparte
    cp #0x03
    jp z, cuartaparte
    cp #0x04
    jp z, quintaparte

    ret

    ;;PRIMERA PARTE   
    primeraparte:
    ld a, #0x00
    ld (flipped_dead), a
 
    ld b,#5
    halt
    dec b
    jr nz, . - 2

    call game_man_getCurrentDirection
    cp #player_right
    jr z, animation_continue

    call game_man_getCurrentDirection
    cp #player_left
    jr nz, animation_random

    call animation_flip_muerte
    ld a, #0x01
    ld (flipped_dead), a
    jr animation_continue

    animation_random:
    call cpct_getRandom_xsp40_u8_asm
     cp #81
     jr nc, animation_continue

     call animation_flip_muerte
    ld a, #0x01
    ld (flipped_dead), a

    animation_continue:
    call render_sys_change_screen
    call render_sys_player
    call render_sys_change_screen
    call render_sys_player

    ld a, (proceso_muerte)
    inc a
    ld (proceso_muerte), a
    ret

    ;;SEGUNDA PARTE
    segundaparte:
    ld b, #20
    halt
    dec b
    jr nz, . - 2

    ld hl, #_spr_dead_0
    ld e_sprite(ix),l
    ld e_sprite+1(ix),h

    call render_sys_change_screen
    call render_sys_player
    call render_sys_change_screen
    call render_sys_player

    ld a, (proceso_muerte)
    inc a
    ld (proceso_muerte), a
    ret

    ;;TERCERA PARTE
    terceraparte:
    ld b, #40
    halt
    dec b
    jr nz, . - 2

    ld hl, #_spr_dead_1
    ld e_sprite(ix),l
    ld e_sprite+1(ix),h
    
    call render_sys_change_screen
    call render_sys_player
    call render_sys_change_screen
    call render_sys_player

    ld a, (proceso_muerte)
    inc a
    ld (proceso_muerte), a
    ret

    ;;CUARTA PARTE
    cuartaparte:
    ld b, #20
    halt
    dec b
    jr nz, . - 2

    ld hl, #_spr_dead_2
    ld e_sprite(ix),l
    ld e_sprite+1(ix),h
    
    call render_sys_change_screen
    call render_sys_player
    call render_sys_change_screen
    call render_sys_player

    ld a, (proceso_muerte)
    inc a
    ld (proceso_muerte), a
    ret

    ;;QUINTA PARTE
    quintaparte:
    ld b, #20
    halt
    dec b
    jr nz, . - 2

    ld hl, #_spr_dead_3
    ld e_sprite(ix),l
    ld e_sprite+1(ix),h   
    call entity_man_getPlayer
    ld a, #0x00
    ld e_contador(ix), a
    
    call render_sys_change_screen
    call render_sys_player
    call render_sys_change_screen
    call render_sys_player

    ld a, #0x00
    ld (proceso_muerte), a          

    call entity_man_resetJugador 

    call game_man_decVida

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;animation_flip_muerte
;;Hacemos el flip a los sprites de player dead
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: hl bc
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
animation_flip_muerte:

    ld hl, #_spr_dead_0
    ld  b, #Height16
    ld  c, #Width8
    call cpct_hflipSpriteM0_asm

    ld hl, #_spr_dead_1
    ld  b, #Height16
    ld  c, #Width8
    call cpct_hflipSpriteM0_asm

    ld hl, #_spr_dead_2
    ld  b, #Height16
    ld  c, #Width8
    call cpct_hflipSpriteM0_asm

    ld hl, #_spr_dead_3
    ld  b, #Height16
    ld  c, #Width8
    call cpct_hflipSpriteM0_asm

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;animation_walking
;;Calculamos el sprite de movimiento que debe utilizar el personaje
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: a bc de hl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
animation_walking:

    ld a, e_vx(ix)
    cp #0
    jp z, quietoenejex

        ld a, #0x02
        ld (no_quieto), a

        call game_man_getDireccionXPlayer
        cp #player_right
        jr z, anim_derecha
        cp #player_firstmoveX
        jr z, anim_derecha

        ;;IZQUIERDA
        ld a, (last_animX)
        cp #anim_right
        jr nz, no_flip_yet

        ;;Si la última animación fue la de la derecha hacemos flip
        call anim_sys_flip_movementX

        no_flip_yet:
        ld a, (sprite_walking_in_moveX)
        cp #paso_izquierda
        jr z, pasoleft

        ;;VA CON PIE DERECHO
        ld hl, #_spr_penguin_3
        ld e_sprite(ix), l
        ld e_sprite+1(ix), h
        ld a, #paso_izquierda
        ld (sprite_walking_in_moveX), a
        ld a, #anim_left
        ld (last_animX), a
        ret

        pasoleft:
        ;;;VA CON PIE IZQUIERDO
        ld hl, #_spr_penguin_5
        ld e_sprite(ix), l
        ld e_sprite+1(ix), h
        ld a, #paso_derecha
        ld (sprite_walking_in_moveX), a
        ld a, #anim_left
        ld (last_animX), a
        ret
  
        ;;DERECHA
        anim_derecha:
        ld a, (last_animX)
        cp #anim_left
        jr nz, no_flip_yet1

        ;;Si la última animación fue la de la izquierda hacemos flip
        call anim_sys_flip_movementX

        no_flip_yet1:        
        ld a, (sprite_walking_in_moveX)
        cp #paso_derecha
        jr z, pasoright

        ;;VA CON PIE IZQUIERDO
        ld hl, #_spr_penguin_5
        ld e_sprite(ix), l
        ld e_sprite+1(ix), h
        ld a, #paso_derecha
        ld (sprite_walking_in_moveX), a
        ld a, #anim_right
        ld (last_animX), a
        ret

        pasoright:
        ;;VA CON PIE DERECHO
        ld hl, #_spr_penguin_3
        ld e_sprite(ix), l
        ld e_sprite+1(ix), h
        ld a, #paso_izquierda
        ld (sprite_walking_in_moveX), a
        ld a, #anim_right
        ld (last_animX), a
        ret

    ;;MOVIMIENTO EN EJE Y
    quietoenejex:
    ld a, e_vy(ix)
    cp #0
    jp z, quietoenejexyenejey

        ld a, #0x02
        ld (no_quieto), a

        call game_man_getDireccionYPlayer
        cp #player_up
        jr z, anim_arriba
        cp #player_firstmoveY
        jr z, anim_arriba

        ;;DOWN
        ld a, (last_animY)
        cp #anim_up
        jr nz, no_flip_yet_down

        ;;Si la última animación fue la de abajo hacemos flip
        call anim_sys_flip_movementY

        no_flip_yet_down:
        ld a, (sprite_walking_in_moveY)
        cp #paso_abajo
        jr z, pasodown

        ;;VA CON PIE DERECHO
        ld hl, #_spr_penguin_2
        ld e_sprite(ix), l
        ld e_sprite+1(ix), h
        ld a, #paso_abajo
        ld (sprite_walking_in_moveY), a
        ld a, #anim_down
        ld (last_animY), a
        ret

        pasodown:
        ;;;VA CON PIE IZQUIERDO
        ld hl, #_spr_penguin_4
        ld e_sprite(ix), l
        ld e_sprite+1(ix), h
        ld a, #paso_arriba
        ld (sprite_walking_in_moveY), a
        ld a, #anim_down
        ld (last_animY), a
        ret
  
        ;;ARRIBA
        anim_arriba:
        ld a, (last_animY)
        cp #anim_down
        jr nz, no_flip_yet3

        ;;Si la última animación fue la de la arriba hacemos flip
        call anim_sys_flip_movementY

        no_flip_yet3:        
        ld a, (sprite_walking_in_moveY)
        cp #paso_arriba
        jr z, pasoup

        ;;VA CON PIE IZQUIERDO
        ld hl, #_spr_penguin_4
        ld e_sprite(ix), l
        ld e_sprite+1(ix), h
        ld a, #paso_arriba
        ld (sprite_walking_in_moveY), a
        ld a, #anim_up
        ld (last_animY), a
        ret

        pasoup:
        ;;VA CON PIE DERECHO
        ld hl, #_spr_penguin_2
        ld e_sprite(ix), l
        ld e_sprite+1(ix), h
        ld a, #paso_abajo
        ld (sprite_walking_in_moveY), a
        ld a, #anim_up
        ld (last_animY), a
        ret

    quietoenejexyenejey:
    call game_man_getCurrentDirection
    cp #player_up
    jr z, miraarriba

    cp #player_right
    jr z, miraderecha

    cp #player_left
    jr z, miraizquierda

    jr miraabajo

    miraarriba:

        ld de, #0x0078
        ld hl, #_spr_penguin_0
        add hl, de
        ex de, hl
        ld hl, #_spr_penguin_0

        ld  b, #Height16
        ld  c, #Width8

        ld e_sprite(ix), l
        ld e_sprite+1(ix), h

        ld a, (sprite_walking_in_useY)
        cp #quieto_abajo
        jr z, flip_to_up

        call game_man_getDireccionYPlayer   
        cp #player_up
        ret z

        call game_man_getDireccionYPlayer
        cp #player_firstmoveY
        ret z

        flip_to_up:
        call cpct_vflipSprite_asm
        ld a, #quieto_arriba
        ld (sprite_walking_in_useY), a

        ret

    miraderecha:

        ld hl, #_spr_penguin_1
        ld  b, #Height16
        ld  c, #Width8

        ld e_sprite(ix), l
        ld e_sprite+1(ix), h

        ld a, (sprite_walking_in_useX)
        cp #quieto_izquierda
        jr z, flip_to_right

        call game_man_getDireccionXPlayer
        cp #player_right
        ret z

        call game_man_getDireccionXPlayer
        cp #player_firstmoveX
        ret z
        
        flip_to_right:
        call cpct_hflipSpriteM0_asm
        ld a, #quieto_derecha
        ld (sprite_walking_in_useX), a

        ret

    miraizquierda:

        ld hl, #_spr_penguin_1

        ld e_sprite(ix), l
        ld e_sprite+1(ix), h

        ld a, (sprite_walking_in_useX)
        cp #quieto_izquierda
        ret z

        ld  b, #Height16 
        ld  c, #Width8

        call cpct_hflipSpriteM0_asm
        ld a, #quieto_izquierda
        ld (sprite_walking_in_useX), a

        ret

    miraabajo:    

        ld de, #0x0078
        ld hl, #_spr_penguin_0
        add hl, de
        ex de, hl
        ld hl, #_spr_penguin_0

        ld e_sprite(ix), l
        ld e_sprite+1(ix), h 

        ld a, (sprite_walking_in_useY)
        cp #quieto_abajo
        ret z

        ld  b, #Height16
        ld  c, #Width8

        call cpct_vflipSprite_asm
        ld a, #quieto_abajo
        ld (sprite_walking_in_useY), a

  ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;anim_sys_flip_movementX
;;Hacemos flip a los sprites de paso del eje X
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: de hl bc
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
anim_sys_flip_movementX:

    ld hl, #_spr_penguin_3
    ld  b, #Height16
    ld  c, #Width8
    call cpct_hflipSpriteM0_asm

    ld hl, #_spr_penguin_5
    ld  b, #Height16
    ld  c, #Width8
    call cpct_hflipSpriteM0_asm

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;anim_sys_flip_movementY
;;Hacemos flip a los sprites de paso del eje Y
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: de hl bc
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
anim_sys_flip_movementY:

    ld de, #0x0078
    ld hl, #_spr_penguin_2
    add hl, de
    ex de, hl
    ld hl, #_spr_penguin_2
    ld  b, #Height16
    ld  c, #Width8
    call cpct_vflipSprite_asm

    ld de, #0x0078
    ld hl, #_spr_penguin_4
    add hl, de
    ex de, hl
    ld hl, #_spr_penguin_4
    ld  b, #Height16
    ld  c, #Width8
    call cpct_vflipSprite_asm

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;anim_sys_reset
;;Reseteamos los datos de las animaciones
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: a de hl bc
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
anim_sys_reset:

    ld a, (flipped_dead)
    cp #0x00
    jr z, continue_reset

        call animation_flip_muerte

        ld a, #0x00
        ld (flipped_dead), a

    continue_reset:
    ld a, (sprite_walking_in_useX)
    cp #quieto_derecha
    jr z, continue_reset1

        ld hl, #_spr_penguin_1
        ld  b, #Height16
        ld  c, #Width8  
        call cpct_hflipSpriteM0_asm     

        ld a, #quieto_derecha
        ld (sprite_walking_in_useX), a

    continue_reset1:
    ld a, (sprite_walking_in_useY)
    cp #quieto_arriba
    jr z, continue_reset2

        ld de, #0x0078
        ld hl, #_spr_penguin_0
        add hl, de
        ex de, hl
        ld hl, #_spr_penguin_0
        ld  b, #Height16
        ld  c, #Width8
        call cpct_vflipSprite_asm

        ld a, #quieto_arriba
        ld (sprite_walking_in_useY), a

    continue_reset2:        

        ld a, #paso_first
        ld (sprite_walking_in_moveX), a
        ld (sprite_walking_in_moveY), a

    ld a, (last_animX)
    cp #anim_right
    jr z, continue_reset5

        call anim_sys_flip_movementX

        ld a, #anim_right
        ld (last_animX), a

    continue_reset5:
    ld a, (last_animY)
    cp #anim_up
    jp z, continue_reset6

        call anim_sys_flip_movementY

        ld a, #anim_up
        ld (last_animY), a

    continue_reset6:
    ld a, #0x00
    ld (no_quieto), a

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;animation_huron
;;Realiza la animación del hurón
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: ix (entidad hurón)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
animation_huron:

    ld a, e_auxiliar(ix)
    cp #0x00
    jr z, huron_sprite2

        ld hl, #_spr_huron_0
        ld a, #0x00
        ld e_auxiliar(ix), a
        jr fin_huron_anim

    huron_sprite2:
    ld hl, #_spr_huron_1
    inc a
    ld e_auxiliar(ix), a

    fin_huron_anim:
    ld e_sprite(ix), l
    ld e_sprite+1(ix), h

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;animation_meta
;;Realiza la animación de haber llegado a la meta
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
animation_meta:

    ;;¿A QUE PARTE VAMOS?
    ld a, (proceso_meta)
    cp #0x00
    jr z, primeraparte1
    cp #0x01
    jr z, segundaparte1
    cp #0x02
    jr z, terceraparte1
    cp #0x03
    jp z, cuartaparte1
    cp #0x04
    jp z, quintaparte1
    cp #0x05
    jp z, sextaparte1

    ret

    primeraparte1:
    ;;Primera vez que entra

    ;;Poner personaje en rosa    
    call animation_sys_disappear_player

    call colision_sys_getMetaColisionada
    ld a, #type_enemigo
    ld e_type(ix), a ;La meta se convierte en un enemigo
    ld hl, #_spr_meta_0 ;Actualizamos el sprite a meta conseguida
    ld e_sprite(ix), l
    ld e_sprite+1(ix), h

    ld a, (proceso_meta)
    inc a
    ld (proceso_meta), a

    ld b, #30
    halt
    dec b
    jr nz, . - 2

    ret

    segundaparte1:
    ;;Segunda vez que entra
    ;;Sprite meta puños
    call animation_sys_disappear_player
    
    call colision_sys_getMetaColisionada
    ld hl, #_spr_meta_1
    ld e_sprite(ix), l
    ld e_sprite+1(ix), h

    ld a, (proceso_meta)
    inc a
    ld (proceso_meta), a

    ld b, #50
    halt
    dec b
    jr nz, . - 2

    ret

    terceraparte1:
    ;;Tercera vez que entra
    ;;Sprite meta normal
    call colision_sys_getMetaColisionada
    ld hl, #_spr_meta_0
    ld e_sprite(ix), l
    ld e_sprite+1(ix), h

    ld a, (proceso_meta)
    inc a
    ld (proceso_meta), a

    ld b, #30
    halt
    dec b
    jr nz, . - 2

    ret

    cuartaparte1:
    ;;Cuarta vez que entra
    ;;Sprite meta puños
    call colision_sys_getMetaColisionada
    ld hl, #_spr_meta_1
    ld e_sprite(ix), l
    ld e_sprite+1(ix), h

    ld a, (proceso_meta)
    inc a
    ld (proceso_meta), a

    ld b, #50
    halt
    dec b
    jr nz, . - 2

    ret

    quintaparte1:
    ;;Quinta vez que entra
    ;;Sprite meta normal definitiva
    call colision_sys_getMetaColisionada
    ld hl, #_spr_meta_0
    ld e_sprite(ix), l
    ld e_sprite+1(ix), h

    ld a, (proceso_meta)
    inc a
    ld (proceso_meta), a

    ld b, #30
    halt
    dec b
    jr nz, . - 2

    ret

    sextaparte1:
    ;;Sexta vez que entra
    ;;Reset de jugador
    call entity_man_resetJugador ;El jugador vuelve al inicio    

    ld a, #0x00
    ld (proceso_meta), a  
    call game_man_setGoAnimationMeta    

    ld b, #10
    halt
    dec b
    jr nz, . - 2

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;animation_sys_disappear_player
;;El personaje recibe una mascara rosa para que no se vea en pantalla
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
animation_sys_disappear_player:

    call entity_man_getPlayer

    ld hl, #_spr_mascara
    ld e_sprite(ix), l
    ld e_sprite+1(ix), h
    call render_sys_player

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;animation_getProcesoMeta
;;Devolvemos la parte por la que vamos de la animación de la meta
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
animation_getProcesoMeta:
    ld a, (proceso_meta)
    ret