;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of PENGUIN CROSS 
;;  Copyright (C) 2020 Francesc Bellido Delgado
;;  Copyright (C) 2020 David Mulero Pérez
;;  Copyright (C) 2020 Wei He
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

.include "manager/entity_man.h.s"
.include "manager/game_man.h.s"
.include "manager/template_man.h.s"
.include "colision_sys.h.s"
.include "music_sys.h.s"
.include "ia_sys.h.s"
.include "cpctelera.h.s"

colision_bool:
	.db 0x00
colisionObject1:
	.dw 0xDEAD
colisionObject2:
	.dw 0xDEAD
meta_colisionada:
	.dw 0xDEAD

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Colision_sys_update
;;Actualiza el sistema de colision
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
colision_sys_update:

	call game_man_getGoAnimationDead
	cp #0x00
	ret nz

	call game_man_getGoAnimationMeta
	cp #0x00
	ret nz

      ld hl, #0xDEAD
      ld (colisionObject1), hl
      ld (colisionObject2), hl

	ld hl, #colision_sys_check_one_colision
	ld  d, #cmp_colider
	call entity_man_forPlayerAndAll ;Comprobamos las colisiones posibles de cada entidad

	ld a, (colision_bool)
	cp #0x00
	jr nz, fin_colision_update

	call entity_man_getPlayer
	ld a, e_fila(ix)
	cp #0x07
	ret c

	call colision_sys_colisionEnemigo ;colisiona con el Mar
	ret

   fin_colision_update:	
   	ld a, #0x00
	ld (colision_bool), a

	ld hl, (colisionObject1)
	call colision_sys_doColisionObjectBehaviour
	ld hl, (colisionObject2)
	call colision_sys_doColisionObjectBehaviour

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Colision_sys_check_one_colision
;;Comprueba la colision del las entidades pasadas por parámetro
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:    ix: entidad 1
;;		iy: entidad 2
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:   
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
colision_sys_check_one_colision:

	call game_man_getGoAnimationDead
	cp #0x00
	ret nz
	
	;Recibimos iy de la función entity_man_forPlayerAndAll
	;La entidad ix es el player
   	ld a, e_fila(ix) ;Comprobamos la fila
   	cp e_fila(iy)
   	ret nz ;Si las filas no son las mismas acabamos

   	ld a,e_x(iy)
   	cp #0x80
   	jr nc, check_one_colision_es_negativo

   	ld a, e_x(ix) ;Se comprueba que el enemigo colisione por la derecha
      dec a			;	x		  x+width   -   x(enemigo) = acarreo				
	add e_w(ix)  	;	___________________		_____
	sub e_x(iy)    	;     |			\		\    \
	ret c			;     |     Jugador     \		\    \
			      ;	___________________		 _____
     check_one_colision_es_negativo:
      ld a, e_x(iy) 
    	dec a		  ;Se comprueba que el enemigo colisione por la izquierda
	add e_w(iy)
	sub e_x(ix)
	ret c

	ld a, e_y(ix) ;Se comprueba que el enemigo colisione por abajo
    	dec a
	add e_h(ix)
	sub e_y(iy)
	ret c

	ld a, e_y(iy)
    	dec a		  ;Se comprueba que el enemigo colisione por arriba
	add e_h(iy)
	sub e_y(ix)
	ret c

	ld a, #type_enemigo
	and e_type(iy)
	cp e_type(iy)
	jr nz, colision_meta ;Si no es de tipo enemigo es la meta (o no)

	call colision_sys_colisionEnemigo
	ld a, #0x01
	ld (colision_bool), a
	ret

    colision_meta: 
    	ld a, #type_meta
	and e_type(iy)
	cp e_type(iy)
	jr nz, colision_plataforma ;Si no es de tipo meta es la plataforma (o no)

    	call colision_sys_colisionMeta
	ld a, #0x01
	ld (colision_bool), a
	ret

    colision_plataforma:
	ld a, #type_plataforma
	and e_type(iy)
	cp e_type(iy)
	jr nz, colision_item ;Si no es de tipo plataforma es item (o no)
	ld a, #0x01
	ld (colision_bool), a

	call colision_sys_setColisionObject
	ret

    colision_item:

	ld a, #type_item
	and e_type(iy)
	cp e_type(iy)
	ret nz
	call colision_sys_colisionItem
	ld a, #0x01
	ld (colision_bool), a
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;colision_sys_colisionEnemigo
;;Mata al jugador
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: 
;		ix: jugador
;		iy: entidad 2
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
colision_sys_colisionEnemigo:

    push ix
	ld a, #0x01
  	call game_man_setGoAnimationDead
	call game_man_getVida ;Devuelve vida restante en registro a
	cp #0x00		    ;Comprobamos si es 0
	jr z, morir
    pop ix

    ret

    morir:
    pop ix
    call entity_man_resetJugador
    call entity_man_set4destroy ;El player muere    	

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;colision_sys_colisionMeta
;;acción al llegar a la meta
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: 
;		ix: jugador
;		iy: entidad 2
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
colision_sys_colisionMeta:

	call game_man_incMeta

	ld (meta_colisionada), iy

	ld a, #0x01
	call game_man_setGoAnimationMeta

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;colision_sys_colisionItem
;;acción al tocar un item
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: 
;		ix: jugador
;		iy: item
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
colision_sys_colisionItem:

	push iy
	pop ix
	call music_sys_reproducirSonidoItem
	ld l, e_behaviour(ix)
	ld h, e_behaviour+1(ix)

	ld bc, #finColisionItem
	jp (hl)
	
    finColisionItem:
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;colision_sys_setColisionObject
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: 
;		iy: entidad
;	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
colision_sys_setColisionObject:

	ld hl, (colisionObject1)
      ld de, #0xDEAD

	ld a, h
	cp d
	jr nz, comprobarObj2
	ld a, l
	cp e
	jr nz, comprobarObj2

    actualizarObj1:
    	ld (colisionObject1), iy
    	ret

    comprobarObj2:
      ld hl, (colisionObject2)
      ld de, #0xDEAD

	ld a, h
	cp d
	ret nz
	ld a, l
	cp e
	ret nz

    actualizarObj2:
    	ld (colisionObject2), iy
    	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;colision_sys_doColisionObjectBehaviour
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: hl object
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
colision_sys_doColisionObjectBehaviour:

      ld de, #0xDEAD

	ld a, h
	cp d
	jr nz, invocarBehaviour
	ld a, l
	cp e
	ret z

    invocarBehaviour:
    	ld a, #sumergir
    	push hl
    	pop ix

   	ld h, e_behaviour+1(ix)
   	ld l, e_behaviour(ix)
	ld bc, #finInvocar
    	push bc ;Guardamos la dirección 'bc' al final de la pila
    	jp (hl)

    finInvocar:
    	ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;colision_sys_getColisionObject1	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: de colisionObject
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
colision_sys_getColisionObject1:

	ld de, (colisionObject1)
	ret 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;colision_sys_getColisionObject2
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: de colisionObject
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
colision_sys_getColisionObject2:

	ld de, (colisionObject2)
	ret 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;colision_sys_getMetaColisionada
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: ix
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
colision_sys_getMetaColisionada:

	ld ix, (meta_colisionada)
	ret