;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of PENGUIN CROSS 
;;  Copyright (C) 2020 Francesc Bellido Delgado
;;  Copyright (C) 2020 David Mulero Pérez
;;  Copyright (C) 2020 Wei He
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

.include "ia_sys.h.s"
.include "cpctelera.h.s"
.include "manager/entity_man.h.s"
.include "../cpct_functions.h.s"
.include "manager/game_man.h.s"
.include "manager/template_man.h.s"
.include "render_sys.h.s"
.include "physics_sys.h.s"
.include "colision_sys.h.s"

contador_huron:
	.db 0xCC
portal1_ptr:
	.db 0xDE, 0xAD
portal2_ptr:
	.db 0xDE, 0xAD

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;ia_sys_update
;;Actualiza la ia de las entidades
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ia_sys_update:

	call game_man_getContadorCongelar
      cp #0
      ret nz

	ld hl, #ia_sys_update_one
	ld d , #cmp_haveIA
	call entity_man_forAll 
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;ia_sys_update_one
;;actualiza la ia de una entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: ix la entidad a actualizar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:a bc 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ia_sys_update_one:

   	ld h, e_behaviour+1(ix)
   	ld l, e_behaviour(ix)
	ld bc, #iaUpdateOne_Next
    	push bc ;Guardamos la dirección 'bc' al final de la pila
    	jp (hl)

  iaUpdateOne_Next:
      ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;ia_sys_behaviour_LeftOrRight
;;comportamiento de ir a la derecha o izquierda y desaparecer
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: ix la entidad a actualizar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:a bc 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ia_sys_behaviour_LeftOrRight:

	;Comprobamos si se sale de la pantalla
	ld b, #screen_width
      ld a, e_x(ix)
      cp b
      ret c

      ld a,#0
      sub e_w(ix)
      ld b, e_x(ix)
      cp b
      ret c
      ;Se ha salido, lo marcamos para destruir 
      call entity_man_set4destroy
      ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;ia_sys_behaviour_Guardian
;;comportamiento de ir a la derecha y a la izquierda
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: ix la entidad a actualizar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:a bc 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ia_sys_behaviour_Guardian:

    push ix
    call entity_man_getPlayer
    push ix
    pop iy ;iy jugador
    pop ix ;ix enemigo

    ;están en la misma fila?
    ld a, e_fila(ix)
    cp e_fila(iy)
    jr nz, noVision

    ;esta el jugador delante del enemigo?
    ld a, e_vx(ix)
    cp #0x80
    jr c, checkDerecha

    ld a, e_x(ix)
    cp e_x(iy)
    jr c, noVision

    ld a, #0x02
    ld e_constContador(ix), a
    ld a, #0x4a
    ld b, #0x85
    call template_man_cambiarColorHuron
    jr checkMovement

   checkDerecha:
       ld a, e_x(ix)
    cp e_x(iy)
    jr nc, noVision

    ld a, #0x02
    ld e_constContador(ix), a
    ld a, #0x4a
    ld b, #0x85
    call template_man_cambiarColorHuron
    jr checkMovement

   noVision:
    ld a, #0x04
    ld e_constContador(ix), a
    ld a, #0x6a
    ld b, #0x95
    call template_man_cambiarColorHuron

   checkMovement:
    ld a, (contador_huron)
    cp #0
    jr nz, checkMovementContador

    ld b, #screen_width
    ld a, #11
    neg
    add b
    ld c, a

    ;Comprobamos si se sale de la pantalla
    ld a, e_x(ix)
    cp c
    ret c

    ;Se ha salido, invertimos la velocidad
    ld a, e_vx(ix)
    neg
    ld e_vx(ix), a
    ld a, #0x01 ; para que se gire ya
    ld e_contador(ix), a

    ;;Giramos primer sprite
    ld hl, #_spr_huron_0
    ld  b, #16
    ld  c, #11
    call cpct_hflipSpriteM0_asm

    ;;Giramos segundo sprite
    ld hl, #_spr_huron_1
    ld  b, #16
    ld  c, #11
    call cpct_hflipSpriteM0_asm

    ret

   checkMovementContador:
    dec a
    ld (contador_huron),a
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;ia_sys_behaviour_Pocion
;;comportamiento de la poción 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: ix la entidad a actualizar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:a 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ia_sys_behaviour_Pocion:

    cpctm_setBorder_asm HW_BRIGHT_RED

    call game_man_incVidaExtra   ;añades vida y si no puedes añadir....
    call entity_man_set4destroy
    call render_sys_update_one

    ;;Tiempo para que se recorra el raster y ponga el borde completo en color rojo
    ld b, #0x10
    halt
    dec b
    jr nz, .-2
    
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;ia_sys_behaviour_Reloj
;;comportamiento del reloj
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: ix la entidad a actualizar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:a 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ia_sys_behaviour_Reloj:

	call game_man_congelar
	call entity_man_set4destroy
	call render_sys_update_one

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;ia_sys_reset_huron
;;Reset huron:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:a 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ia_sys_reset_huron:

	ld a,#0xCC
	ld (contador_huron),a

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;ia_sys_behaviour_portal
;;Behaviour de Portal
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: ix la entidad a actualizar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ia_sys_behaviour_Portal:

	call entity_man_set4destroy
	call render_sys_update_one

	push ix
	pop de
	ld hl,(portal1_ptr)
	ld a, h
	cp d
	jr nz, portal2 ;Si h es diferente 
	ld a, l
	cp e
	jr nz, portal2 ;Si l es diferente 

	ld hl,(portal2_ptr)
	push hl
	pop ix
	jr portaldestino

	portal2:

	ld hl,(portal1_ptr)
	push hl
	pop ix

	portaldestino:


	call entity_man_set4destroy
	call render_sys_update_one
	
	ld b, e_x(ix)
	ld c, e_y(ix)
	
	call physics_sys_movePlayer

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;ia_sys_setPortal1Pointer
;;Set pointer
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: de pointer
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ia_sys_setPortal1Pointer:

    ld (portal1_ptr),de

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;ia_sys_setPortal2Pointer
;;Set pointer
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: de pointer
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ia_sys_setPortal2Pointer:

    ld (portal2_ptr),de

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;ia_sys_behaviour_plataforma
;;Behaviour de plataforma hundible
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: ix la entidad a actualizar
;;        a sumergir/reset/nada
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ia_sys_behaviour_plataforma:

	push af
	call ia_sys_behaviour_LeftOrRight
	pop af

	;a == CC -> sumergir
	cp #sumergir
	jr nz, resetPlataforma

	call ia_sys_behaviour_sumergir
	ret 

    resetPlataforma:
    	ld a, e_auxiliar(ix)
    	cp #0x80
    	ret z

    	push ix
    	pop hl
    	call colision_sys_getColisionObject1

    	ld a, h
    	cp d
    	jr nz, checkEstado2
    	ld a, l
    	cp e
    	ret z

    checkEstado2:
    	call colision_sys_getColisionObject2
    	ld a, h
    	cp d
    	jr nz, resetear
    	ld a, l
    	cp e
    	ret z
    
    resetear:
	call ia_sys_behaviour_resetSumergir
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;ia_sys_behaviour_sumergir
;;HUndir la plataforma
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: ix la entidad a actualizar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ia_sys_behaviour_sumergir:

	ld a, e_auxiliar(ix)
	dec a
	ld e_auxiliar(ix), a

	cp #0x41
	ret nc

	cp #0x00
	jr nz, cambiarSpriteSumergir

	call entity_man_set4destroy
	call render_sys_update_one
	ret

    cambiarSpriteSumergir:
	cp #0x40
	jr nz, cambiarSpriteSumergir2

	ld a, e_constContador(ix)
	cp #4
	jr nc, spriteSumergido0

	ld hl, #_spr_maderahundida_0 
	jr cambiarSpriteSumergirFinal

    spriteSumergido0:
	ld hl, #_spr_sumergido_0 
	jr cambiarSpriteSumergirFinal

	cambiarSpriteSumergir2:
	cp #0x26
	jr nz, cambiarSpriteSumergir3

	ld a, e_constContador(ix)
	cp #4
	jr nc, spriteSumergido1

	ld hl, #_spr_maderahundida_1 
	jr cambiarSpriteSumergirFinal

    spriteSumergido1:
	ld hl, #_spr_sumergido_1 
	jr cambiarSpriteSumergirFinal

	cambiarSpriteSumergir3:
	cp #0x12
	ret nz

	ld a, e_constContador(ix)
	cp #4
	jr nc, spriteSumergido2

	ld hl, #_spr_maderahundida_2 
	jr cambiarSpriteSumergirFinal

    spriteSumergido2:
	ld hl, #_spr_sumergido_2 

	cambiarSpriteSumergirFinal:
	ld e_sprite(ix), l
	ld e_sprite+1(ix), h

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;ia_sys_behaviour_resetSumergir
;;HUndir la plataforma
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: ix la entidad a actualizar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ia_sys_behaviour_resetSumergir:

	ld a, e_auxiliar(ix)
	cp #0x80
	ret z
	inc a
	ld e_auxiliar(ix), a

	;;reseteamos sprites poco a poco
	cp #0x18
	ret c

	cp #0x40
	jr c, cambiarSpriteResetSumergir2
	ret nz

	ld a, e_constContador(ix)
	cp #4
	jr nc, resetApriteSumergido

	ld hl, #_spr_madera 
	jr cambiarSpriteResetSumergirFinal

    resetApriteSumergido:
	ld hl, #_spr_hielo 
	jr cambiarSpriteResetSumergirFinal

    cambiarSpriteResetSumergir2:
	cp #0x26
	jr c, cambiarSpriteResetSumergir3
	ret nz

	ld a, e_constContador(ix)
	cp #4
	jr nc, resetApriteSumergido0

	ld hl, #_spr_maderahundida_0 
	jr cambiarSpriteResetSumergirFinal

    resetApriteSumergido0:
	ld hl, #_spr_sumergido_0 
	jr cambiarSpriteResetSumergirFinal

    cambiarSpriteResetSumergir3:
	cp #0x12
	ret nz

	ld a, e_constContador(ix)
	cp #4
	jr nc, resetApriteSumergido1

	ld hl, #_spr_maderahundida_1 
	jr cambiarSpriteResetSumergirFinal

    resetApriteSumergido1:
	ld hl, #_spr_sumergido_1 

    cambiarSpriteResetSumergirFinal:
	ld e_sprite(ix), l
	ld e_sprite+1(ix), h
	ret