;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of PENGUIN CROSS 
;;  Copyright (C) 2020 Francesc Bellido Delgado
;;  Copyright (C) 2020 David Mulero Pérez
;;  Copyright (C) 2020 Wei He
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

.include "input_sys.h.s"
.include "../cpct_functions.h.s"
.include "cpctelera.h.s"
.include "manager/entity_man.h.s"
.include "manager/template_man.h.s"
.include "manager/menu_man.h.s"
.include "manager/game_man.h.s"
.include "manager/state_man.h.s"
.include "music_sys.h.s"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;input_sys_init
;;Inicialización del sistema de input
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
input_sys_init:

	ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;input_sys_update
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
input_sys_update:

	ld hl, #input_sys_check_key
	ld  d, #cmp_control
	call entity_man_forAll ;Comprobamos todos los posibles controles

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;input_sys_check_key
;;Comprueba si se ha pulsado una tecla para actualizar la velocidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: ix la entidad a actualizar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: a hl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
input_sys_check_key:

	call cpct_scanKeyboard_f_asm

	ld hl, #Key_Esc
	call cpct_isKeyPressed_asm
	jr z, continueKeys

	call game_man_escape

	continueKeys:
	ld a, #0x00
	cp e_contador(ix)
	jp nz, finish_key ;Si el contador aún no es cero acabamos

	ld a, e_fila(ix)
	call game_man_setLastFila

	ld a, #00 ;Inicialmente ponemos las velocidades a cero
	ld e_vx(ix), a
	ld e_vy(ix), a

	;CURSOR UP
	ld hl, #Key_CursorUp
	call cpct_isKeyPressed_asm
	jr nz, esUp

	ld hl, #Key_Q
	call cpct_isKeyPressed_asm
	jr nz, esUp

	ld hl, #Joy0_Up
	call cpct_isKeyPressed_asm
	jr z, down_key
   
    esUp:
	ld a, #player_up
	call game_man_newDirection

	ld a, #0xFC
	ld e_vy(ix), a

	ld a, #0x0D
	cp e_fila(ix)
	jr z, up_keyseguir ;Comprobamos si esta en la última fila

	ld a, e_fila(ix)
	call game_man_setLastFila 
	inc e_fila(ix)

    up_keyseguir:
	call game_man_getDireccionYPlayer	
	cp #player_up
	jp z, setcont

  	ld a, #player_up
   	call game_man_setDireccionYPlayer

	jp setcont
    
      ;CURSOR DOWN
    down_key:
    	ld hl, #Key_CursorDown
	call cpct_isKeyPressed_asm
	jr nz, esAbajo

    	ld hl, #Key_A
	call cpct_isKeyPressed_asm
	jr nz, esAbajo

	ld hl, #Joy0_Down
	call cpct_isKeyPressed_asm
	jr z, left_key   

    esAbajo:
	ld a, #player_down
	call game_man_newDirection

	ld a, #0x04
	ld e_vy(ix), a

	ld a, #0x00
	cp e_fila(ix)
	jr z, down_keyseguir ;Comprobamos si esta en la última fila
	
	ld a, e_fila(ix)
	call game_man_setLastFila
	dec e_fila(ix) ;Si no está en la última fila restamos uno
	jr down_filanoacero

	down_keyseguir:
	call game_man_getDireccionYPlayer	
	cp #player_firstmoveY
	jr nz, down_filanoacero

	ld a, #0x00
    	call game_man_setLastFila

    	down_filanoacero:
	call game_man_getDireccionYPlayer	
	cp #player_down
	jp z, setcont

  	ld a, #player_down
  	call game_man_setDireccionYPlayer

	jp setcont

	;CURSOR LEFT
    left_key:
    	ld hl, #Key_CursorLeft
	call cpct_isKeyPressed_asm
	jr nz, esLeft

    	ld hl, #Key_O
	call cpct_isKeyPressed_asm
	jr nz, esLeft

	ld hl, #Joy0_Left
	call cpct_isKeyPressed_asm
	jr z, right_key 
    
    esLeft:
	ld a, #player_left
	call game_man_newDirection

	ld a, #0xFE
	ld e_vx(ix), a

	ld a, e_fila(ix)
	call game_man_setLastFila	

	call game_man_getDireccionXPlayer
	cp #player_left
	jp z, setcont

  	ld a, #player_left
   	call game_man_setDireccionXPlayer

	jp setcont

	;CURSOR RIGHT
    right_key:
    	ld hl, #Key_CursorRight
	call cpct_isKeyPressed_asm
	jr nz, esRight

    	ld hl, #Key_P
	call cpct_isKeyPressed_asm
	jr nz, esRight

	ld hl, #Joy0_Right
	call cpct_isKeyPressed_asm
	jr z, finish_key
    
    esRight:
	ld a, #player_right
	call game_man_newDirection

	ld a, #0x02
	ld e_vx(ix), a

	ld a, e_fila(ix)
	call game_man_setLastFila

	call game_man_getDireccionXPlayer
	cp #player_right
	jr z, setcont	

  	ld a, #player_right
  	call game_man_setDireccionXPlayer

	setcont: ;Ponemos un tiempo de espera hasta la siguiente entrada de teclado
	ld a,#4
	ld e_contador(ix), a
	call music_sys_reproducirSonidoPlayer

    finish_key:
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;input_sys_menu_update
;;Comprueba si se ha pulsado una tecla
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: a hl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
input_sys_menu_update:

	call cpct_scanKeyboard_f_asm

	;CURSOR UP
	ld hl, #Key_CursorUp
	call cpct_isKeyPressed_asm
	jr nz, menuEsUp

	ld hl, #Key_Q
	call cpct_isKeyPressed_asm
	jr nz, menuEsUp

	ld hl, #Joy0_Up
	call cpct_isKeyPressed_asm
	jr z, menu_down_key 
   
   menuEsUp:
    	call menu_man_decOpcion
    	ret

      ;CURSOR DOWN
   menu_down_key:
    	ld hl, #Key_CursorDown
	call cpct_isKeyPressed_asm
	jr nz, menuEsAbajo

    	ld hl, #Key_A
	call cpct_isKeyPressed_asm
	jr nz, menuEsAbajo

	ld hl, #Joy0_Down
	call cpct_isKeyPressed_asm
	jr z, menu_enter_key
   
   menuEsAbajo:
	call menu_man_incOpcion
	ret

	;ENTER
   menu_enter_key:
    	ld hl, #Key_Return
	call cpct_isKeyPressed_asm
	jr nz, menuEsEnter

	ld hl, #Joy0_Fire1
	call cpct_isKeyPressed_asm
	jr nz, menuEsEnter

    	ld hl, #Key_Space
	call cpct_isKeyPressed_asm
	ret z

   menuEsEnter:
	call menu_man_pulsarOpcion
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;input_sys_controls_update
;;Comprueba si se ha pulsado una tecla
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: a hl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
input_sys_controls_update:

    call cpct_scanKeyboard_f_asm

    ;ESCAPE
    ld hl, #Key_Esc
    call cpct_isKeyPressed_asm
    jr nz, gotostate

    ret

    gotostate:
    ld a, #state_menu
    call state_man_setStatus
      
    ld d,#1
    ld e, #76
    call music_sys_reproducirNota

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;input_sys_gameover
;;Comprueba si se ha pulsado una tecla
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: a hl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
input_sys_gameover:

    call cpct_scanKeyboard_f_asm

    ;ESCAPE
    ld hl, #Key_Esc
    call cpct_isKeyPressed_asm
    jr nz, gotostate1

    ;ESPACIO
    ld hl, #Key_Space
    call cpct_isKeyPressed_asm
    jr nz, gotostate1

    ;INTRO
    ld hl, #Key_Return
    call cpct_isKeyPressed_asm
    jr nz, gotostate1

    ret

    gotostate1:
    ld a, #state_menu
    call state_man_setStatus   

    ret