;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of PENGUIN CROSS 
;;  Copyright (C) 2020 Francesc Bellido Delgado
;;  Copyright (C) 2020 David Mulero Pérez
;;  Copyright (C) 2020 Wei He
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

.include "music_sys.h.s"
.include "../cpct_functions.h.s"
.include "manager/template_man.h.s"
.include "manager/entity_man.h.s"

efecto:
    .db 0x00
tiempo_efecto:
    .db 0x00
last_nota:
    .db 0xFF
nota:
    .db 0x00

efecto_1:
    .db 76, 79, 81

efecto_2:
    .db 71, 74, 72, 74, 72, 76

efecto_3:
    .db 60, 60, 60, 64

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Music_sys_reproducirEfecto
;;Reproducir efecto
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: d instrumento
;;       e nota
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
music_sys_init:

    push af
    push bc
    push de

      ld de, #__sfx
      call cpct_akp_SFXInit_asm

      ld de, #__musica
      call cpct_akp_musicInit_asm

    pop de
    pop bc
    pop af

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Music_sys_reproducirEfecto
;;Reproducir efecto
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: d instrumento
;;       e nota
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
music_sys_reproducirMusica:

      push af
      push bc
      push de
      push hl

      ld de, #__musica
      call cpct_akp_musicInit_asm

      pop hl
      pop de
      pop bc
      pop af

      ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Music_sys_reproducirEfecto
;;Reproducir efecto
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: d instrumento
;;       e nota
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
music_sys_pararMusica:

      push af
      push bc
      push de
      push hl

      ld de, #__vacio
      call cpct_akp_musicInit_asm

      pop hl
      pop de
      pop bc
      pop af

      ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Music_sys_reproducirNota
;;Reproducir efecto
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: d instrumento
;;       e nota
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
music_sys_reproducirNota:


    ;(1B L ) sfx_num Number of the instrument in the SFX Song (>0), same as the number given to the instrument in Arkos Tracker.
    ;(1B H ) volume  Volume [0-15], 0 = off, 15 = maximum volume.
    ;(1B E ) note    Note to be played with the given instrument [0-143]
    ;(1B D ) speed   Speed (0 = As original, [1-255] = new Speed (1 is fastest))
    ;(2B BC) inverted_pitch  Inverted Pitch (-0xFFFF -> 0xFFFF).  0 is no pitch.  The higher the pitch, the lower the sound.
    ;(1B A ) channel_bitmask Bitmask representing channels to use for reproducing the sound (Ch.A = 001 (1), Ch.B = 010 (2), Ch.C = 100 (4))
 
    ld l, d
    ld h, #12
    ld d, #0
    ld bc, #0
    ld a, #0%00110000
    call cpct_akp_SFXPlay_asm

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Music_sys_reproducirEfecto
;;Reproducir efecto
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: a id efecto
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
music_sys_reproducirEfecto:

    cp #1
    jr nz,efectoSeguir
    ld (efecto),a
    ld a, #62
    ld (tiempo_efecto),a

    ret

    efectoSeguir:
    cp #2
    jr nz,efectoSeguir2
    ld (efecto),a
    ld a, #78
    ld (tiempo_efecto),a

    efectoSeguir2:
    cp #3
    ret nz
    ld (efecto),a
    ld a, #0xD8
    ld (tiempo_efecto),a

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Music_sys_reproducirEfecto
;;Reproducir efecto
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
music_sys_actualizarTiempo:

    ld a,(tiempo_efecto)

    cp #0
    ret z

    dec a
    ld (tiempo_efecto),a

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Music_sys_reproducirEfecto
;;Reproducir efecto
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
music_sys_actualizarEfecto:

    ld a,(efecto)
    cp #0
    ret z

    cp #1
    jr nz, actualizarTiempoE2
    call music_sys_efectoMeta
    jr actualizarEfectoFinal

    actualizarTiempoE2:
    cp #2
    jr nz, actualizarTiempoE3
    call music_sys_efectoNextLevel
    jr actualizarEfectoFinal

    actualizarTiempoE3:
    cp #3
    jr nz, actualizarEfectoFinal
    call music_sys_efecto321

    actualizarEfectoFinal:

    call music_sys_actualizarNotas
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Music_sys_reproducirEfecto
;;Reproducir efecto
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
music_sys_efectoMeta:
    ld d, #2
    ld a,(tiempo_efecto)

    cp #24 ;24
    jr nz,efectoMetaSeguir
    ld a, #0
    jr efectoMetaFinal

    efectoMetaSeguir:
    cp #16 
    jr nz,efectoMetaSeguir2
    ld a, #1
    jr efectoMetaFinal

    efectoMetaSeguir2:
    cp #8
    jr nz,efectoMetaSeguir3
    ld a, #2
    jr efectoMetaFinal

    efectoMetaSeguir3:
    cp #4
    ret nc
    ld a,#0
    ld (efecto),a
    ld a,#0xFF
    ld (last_nota),a
    ld (nota),a
    ret

    efectoMetaFinal:
    ld (nota),a
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Music_sys_reproducirEfecto
;;Reproducir efecto
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
music_sys_efectoNextLevel:
    ld d, #2
    ld a,(tiempo_efecto)

    cp #64 ;24
    jr nz,efectoNextLevelSeguir
    ld a, #0
    jr efectoNextLevelFinal

    efectoNextLevelSeguir:
    cp #56
    jr nz,efectoNextLevelSeguir2
    ld a, #1
    jr efectoNextLevelFinal

    efectoNextLevelSeguir2:
    cp #48
    jr nz,efectoNextLevelSeguir3
    ld a, #2
    jr efectoNextLevelFinal

    efectoNextLevelSeguir3:
    cp #40
    jr nz,efectoNextLevelSeguir4
    ld a, #3
    jr efectoNextLevelFinal

    efectoNextLevelSeguir4:
    cp #24
    jr nz,efectoNextLevelSeguir5
    ld a, #4
    jr efectoNextLevelFinal

    efectoNextLevelSeguir5:
    cp #16
    jr nz,efectoNextLevelSeguir6
    ld a, #5
    jr efectoNextLevelFinal

    efectoNextLevelSeguir6:
    cp #8
    ret nc
    ld a,#0
    ld (efecto),a
    ld a,#0xFF
    ld (last_nota),a
    ld (nota),a
    ret

    efectoNextLevelFinal:
    ld (nota),a
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Music_sys_reproducirEfecto
;;Reproducir efecto
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
music_sys_efecto321:
    ld d, #1
    ld a,(tiempo_efecto)

    cp #0xD7
    jr nz,efecto321Seguir
    ld a, #0
    jr efecto321Final

    efecto321Seguir:
    cp #0xA8
    jr nz,efecto321Seguir2
    ld a, #1
    jr efecto321Final

    efecto321Seguir2:
    cp #0x68
    jr nz,efecto321Seguir3
    ld a, #2
    jr efecto321Final

    efecto321Seguir3:
    cp #0x20
    jr nz,efecto321Seguir4
    ld a, #3
    jr efecto321Final

    efecto321Seguir4:
    cp #8
    ret nc
    ld a,#0
    ld (efecto),a
    ld (tiempo_efecto),a
    ld a,#0xFF
    ld (last_nota),a
    ld (nota),a
    ret

    efecto321Final:
    ld (nota),a
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Music_sys_reproducirEfecto
;;Reproducir efecto
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
music_sys_actualizarNotas:

    ld a, (nota)
    ld c,a
    ld a,(last_nota)

    cp c
    ret z

    ld a, (efecto)
    cp #0
    ret z
    cp #1
    jr nz, actualizarNotas2
    ld hl, #efecto_1
    jr actualizarNotasFinal

    actualizarNotas2:
    cp #2
    jr nz,actualizarNotas3
    ld hl, #efecto_2
    jr actualizarNotasFinal

    actualizarNotas3:
    cp #3
    ret nz
    ld hl, #efecto_3

    actualizarNotasFinal:
    ld b,#0
    add hl,bc
    ld e, (hl)

    ld a,(nota)
    ld (last_nota),a
    call music_sys_reproducirNota

    ret  

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Music_sys_reproducirNota
;;Reproducir efecto
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: d instrumento
;;       e nota
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
music_sys_reproducirSonidoPlayer:

    push af
    push bc
    push de
    push hl
    push ix
    push iy

    ;(1B L ) sfx_num Number of the instrument in the SFX Song (>0), same as the number given to the instrument in Arkos Tracker.
    ;(1B H ) volume  Volume [0-15], 0 = off, 15 = maximum volume.
    ;(1B E ) note    Note to be played with the given instrument [0-143]
    ;(1B D ) speed   Speed (0 = As original, [1-255] = new Speed (1 is fastest))
    ;(2B BC) inverted_pitch  Inverted Pitch (-0xFFFF -> 0xFFFF).  0 is no pitch.  The higher the pitch, the lower the sound.
    ;(1B A ) channel_bitmask Bitmask representing channels to use for reproducing the sound (Ch.A = 001 (1), Ch.B = 010 (2), Ch.C = 100 (4))
   
    ld a, e_fila(ix)
    cp #7
    jr nc, sonidoPlataforma
    ld e, #64
    jr sonidoPlayerSeguir

    sonidoPlataforma:
    ld e, #62

    sonidoPlayerSeguir:

    ld l, #3
    ld h, #7
    ld d, #0
    ld bc, #0
    ld a, #0%00000100
    call cpct_akp_SFXPlay_asm

    pop iy
    pop ix
    pop hl
    pop de
    pop bc
    pop af

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;music_sys_reproducirSonidoItem
;;Reproducir efecto de coger un item
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: d instrumento
;;       e nota
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
music_sys_reproducirSonidoItem:

    push af
    push bc
    push de
    push hl
    push ix
    push iy

    ;(1B L ) sfx_num Number of the instrument in the SFX Song (>0), same as the number given to the instrument in Arkos Tracker.
    ;(1B H ) volume  Volume [0-15], 0 = off, 15 = maximum volume.
    ;(1B E ) note    Note to be played with the given instrument [0-143]
    ;(1B D ) speed   Speed (0 = As original, [1-255] = new Speed (1 is fastest))
    ;(2B BC) inverted_pitch  Inverted Pitch (-0xFFFF -> 0xFFFF).  0 is no pitch.  The higher the pitch, the lower the sound.
    ;(1B A ) channel_bitmask Bitmask representing channels to use for reproducing the sound (Ch.A = 001 (1), Ch.B = 010 (2), Ch.C = 100 (4))
    
    ld e, #76
    ld l, #1
    ld h, #13
    ld d, #0
    ld bc, #0
    ld a, #0%00000100
    call cpct_akp_SFXPlay_asm

    ld e, #88
    ld l, #2
    ld h, #13
    ld d, #0
    ld bc, #0
    ld a, #0%00000100
    call cpct_akp_SFXPlay_asm

    pop iy
    pop ix
    pop hl
    pop de
    pop bc
    pop af

    ret


