;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of PENGUIN CROSS 
;;  Copyright (C) 2020 Francesc Bellido Delgado
;;  Copyright (C) 2020 David Mulero Pérez
;;  Copyright (C) 2020 Wei He
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

.include "physics_sys.h.s"
.include "manager/entity_man.h.s"
.include "../cpct_functions.h.s"
.include "input_sys.h.s"
.include "manager/game_man.h.s"
.include "render_sys.h.s"
.include "animation_sys.h.s"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;physics_sys_update
;;Actualiza la fisica de las entidades
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: a contador congelar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
physics_sys_update:

	ld hl, #physics_sys_update_one
	ld d , #cmp_movable
	call entity_man_forAll ;Realizamos los movimientos de cada entidad

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;physics_sys_update
;;Actualiza la fisica de una entidades
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: ix la entidad a actualizar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:a bc 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
physics_sys_update_one:

     call entity_man_decrementarContador ;Decrementos la espera del jugador

     ld  a, e_type(ix)
     ld  b, #type_player 
     and b
     cp  b 
     jr nz, enemigos

    ;;JUGADOR

     call game_man_getGoAnimationDead
     cp #0x00
     ret nz

     call game_man_getGoAnimationMeta
     cp #0x00
     ret nz

     ;;Calculamos la nueva velocidad Y, que solo entra el jugador
     ld  a, e_y(ix)
     ld  b, e_vy(ix)
     add b

     ;Comprobamos si se sale de la pantalla por eje Y
     ld c, #screen_height - 30
     cp c
     jr nc, finish

     ld c, #16
     cp c
     jr c, finish

     ;No se ha salido, actualizamos la velocidad
     ld e_y(ix), a

     ;;Calculamos la nueva velocidad X, que solo entra el jugador
     ld a, e_x(ix)
     ld b, e_vx(ix)
     add b

     ;Comprobamos si se sale de la pantalla por eje X
     ld c, #screen_width - 7
     cp c
     jr nc, finish

     ;Actualizo la posición respecto a vx
     ld e_x(ix), a

     ;Cambio sprite dependiendo de la velocidad y la direccíón
     call animation_walking

     jr finish

   ;;Solo entran los enemigos o plataforma
   enemigos:

     call game_man_getContadorCongelar
     cp #0
     ret nz
     ;Calculamos la nueva velocidad X
     ld a, #0x00
     cp e_contador(ix)
     jr nz, finish

     ld a, e_x(ix)
     ld b, e_vx(ix)
     add b
     
     ld e_x(ix), a

     ;;Comprobamos que es un hurón
     ld a, e_fila(ix)
     cp #0x06
     ret nz
     call animation_huron

    finish:
     ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;physics_sys_movePlayer
;;Actualiza la fisica de una entidades
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: b: x
;;       c: y
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:a bc 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
physics_sys_movePlayer:

   call entity_man_getPlayer

   ld e_x(ix),b
   ld e_y(ix),c

   ld a, #0x00     ;VX,VY
   ld e_vx(ix), a    
   ld e_vy(ix), a  

   ld a, #0x04
   ld e_contador(ix), a

   ld a,c
   rrca 
   rrca 
   rrca 
   rrca 
   ld c,a

   ld a,#14
   sub a,c
   ld e_fila(ix),a
   call render_sys_borrado_player
   call render_sys_player
   call render_sys_change_screen
   call render_sys_borrado_player
   call render_sys_player
   call render_sys_change_screen
   ld a, e_fila(ix)
   call game_man_setLastFila
   ret