;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of PENGUIN CROSS
;;  Copyright (C) 2020 Francesc Bellido Delgado
;;  Copyright (C) 2020 David Mulero Pérez
;;  Copyright (C) 2020 Wei He
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

 .include "cpctelera.h.s"
 .include "render_sys.h.s"
 .include "manager/entity_man.h.s"
 .include "cpct_functions.h.s"
 .include "../manager/entity_man.h.s"
 .include "../manager/template_man.h.s"
 .include "../manager/game_man.h.s"
 .include "animation_sys.h.s"

string_time:
  .asciz "TIME"

string_nivel:
  .asciz "LEVEL    "

back_buffer:
  .db #0x00, #0xC0

font_array:
  .dw _spr_font_00
  .dw _spr_font_01
  .dw _spr_font_02
  .dw _spr_font_03
  .dw _spr_font_04
  .dw _spr_font_05
  .dw _spr_font_06
  .dw _spr_font_07
  .dw _spr_font_08
  .dw _spr_font_09
  .dw _spr_font_10
  .dw _spr_font_11
  .dw _spr_font_12
  .dw _spr_font_13
  .dw _spr_font_14
  .dw _spr_font_15
  .dw _spr_font_16
  .dw _spr_font_17
  .dw _spr_font_18
  .dw _spr_font_19
  .dw _spr_font_20
  .dw _spr_font_21
  .dw _spr_font_22
  .dw _spr_font_23
  .dw _spr_font_24
  .dw _spr_font_25
  .dw _spr_font_26
  .dw _spr_font_27
  .dw _spr_font_28
  .dw _spr_font_29
  .dw _spr_font_30
  .dw _spr_font_31
  .dw _spr_font_32
  .dw _spr_font_33
  .dw _spr_font_34
  .dw _spr_font_35
  .dw _spr_font_36
  .dw _spr_font_37
  .dw _spr_font_38
  .dw _spr_font_39

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Render_sys_set_crtc
;;Modificar parámetros CRTC
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:  d: registro CRTC, 
;;        e: Valor a añadir
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: c hl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_sys_set_crtc:

  ld bc, #0xBC00     ;Posición de memoria en la que se alojan las opciones del modulo CRTC
  out (c), d         ;Establecer el puerto en función del registro a usar
  inc b
  out (c), e         ;Enviamos el valor que queremos modificar

  ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Render_sys_init
;;Inicialización del render
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: c hl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_sys_init:

  ld c, #0
  call cpct_setVideoMode_asm

  ld de, #0x0120 ;Registro 1
  call render_sys_set_crtc
  ld de, #0x022a ;Registro 2
  call render_sys_set_crtc
  ld de, #0x0620 ;Registro 6
  call render_sys_set_crtc
  ld de, #0x0722 ;Registro 7
  call render_sys_set_crtc

  ld hl, #_main_palette
  ld de, #16
  call cpct_setPalette_asm
  cpctm_setBorder_asm HW_BLACK

  ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;render_sys_update
;;Actualiza la fisica de las entidades
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_sys_update:

   ;;Si no está en la animación de meta hacemos borrado
   call game_man_getGoAnimationMeta
   cp #0x00
   jr z, borradoplayer

      ;;Si está en la animación de meta y el proceso por el que va es menor que 3 borramos
      call animation_getProcesoMeta
      cp #0x03
      jr nc, jumpborrado

   borradoplayer:
   call render_sys_borrado_player

   jumpborrado:
   call render_sys_dibujar_transicion

   ld hl, #render_sys_update_one
   ld d , #cmp_render
   ld e , #type_plataforma
   call entity_man_forAllMatching2

   ld e , #type_item
   call entity_man_forAllMatching2

   ld e , #type_enemigo
   call entity_man_forAllMatching2

   ld e , #type_meta
   call entity_man_forAllMatching2

   call game_man_getGoAnimationDead
   cp #0x00
   jr z, jumpmuerte

   call entity_man_getPlayer
   call animation_sys_muerte
   ret

   jumpmuerte:
   call game_man_getGoAnimationMeta
   cp #0x00
   jr z, jumpmeta

   call animation_meta
   ret

   jumpmeta:
   call render_sys_player

   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;render_sys_update_one
;;Actualiza la fisica de una entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: ix la entidad a actualizar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:a bc de hl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_sys_update_one:

   ld a, e_type(ix)
   ld b, #type_destroy
   and b
   cp b
   jr nz, render_comprobar ;Si no es del tipo destruir avanzamos

   ld b, #2

 borrado_loop:
   push bc
   ld d, e_lastPtr+1(ix)
   ld e, e_lastPtr(ix)
   ld c, e_w(ix)
   ld b, e_h(ix)

   ld a, e_fila(ix)
   cp #0x07
   jr nc, renderoneblanco1
   ld a, #0xC3
   jr renderoneazulseguir

  renderoneblanco1:
   cp #0x0D
   jr c, renderoneazul1
   ld a, #0xC3
   jr renderoneazulseguir

  renderoneazul1:
   ld a, #0x3C

  renderoneazulseguir:
   push af
   ld a, #0x41
   sub e_x(ix)
   sub e_w(ix)
   cp #0x80
   jr c,renderdrawSolidBoxNormalLeft
   add e_w(ix)
   ld c,a
   jr renderdrawSolidBoxNormalSeguir

  renderdrawSolidBoxNormalLeft:

   ld a, e_x(ix)
   cp #0x80
   jr c,renderdrawSolidBoxNormalSeguir

   ld a,e_x(ix)
   add e_w(ix)
   cp #1
   jr nc,renderdrawSolidBoxNoEsNeg
   ld a,#1

  renderdrawSolidBoxNoEsNeg:
   ld c,a      ;ponemos en width solo lo que necesitamos borrar

   ld b,e_h(ix)

  renderdrawSolidBoxNormalSeguir:
   pop af
   call render_sys_drawSolidBox

   ld a, e_lastPtr2(ix)
   ld e_lastPtr(ix),a
   ld a, e_lastPtr2+1(ix)
   ld e_lastPtr+1(ix),a

   pop af
   dec a
   ld b, a
   ret z

   jr borrado_loop

  render_comprobar:

   ld a, e_type(ix)   ;Comprobamos que es un personaje/hud para que no lo renderice
   ld b, #type_player
   and b
   cp b
   ret z

   ld c, e_x(ix)
   ld b, e_y(ix)
   ld de, (back_buffer) ;Inicio de la pantalla -> #CPCT_VMEM_START_ASM
   call render_sys_getScreenPointer

  dibuja: ;Dibujamos en la nueva posición

   ld a, e_lastPtr(ix)
   ld e_lastPtr2(ix),a
   ld a, e_lastPtr+1(ix)
   ld e_lastPtr2+1(ix),a


   ld d, e_sprite+1(ix)
   ld e, e_sprite(ix)
   ex de, hl ;Guardamos en 'hl' el sprite y en 'de' la posición
   ld c, e_w(ix)
    
   ld e_lastPtr(ix), e
   ld e_lastPtr+1(ix), d

   ld a, e_type(ix)
   ld b, #type_item
   cp b
   jr z, render_item ;Si no es del tipo destruir avanzamos

   ld a, #0x40
   sub e_x(ix)
   sub e_w(ix)
   cp #0x80
   jr c,renderClippedLeft

   add e_w(ix)
   ld c,a
   ld a,e_w(ix)
   sub c
   ld b,e_h(ix)
   call render_sys_drawSpriteClipped

   ret

   renderClippedLeft:
   ld a, e_x(ix)
   cp #0x80
   jr c,renderNormal

   ld a,e_x(ix)
   add e_w(ix)
   cp #1
   jr nc,drawSpriteNoZero
   ld a,#1
   drawSpriteNoZero:
   ld c,a
   ld a,e_x(ix)

   ld b,e_h(ix)
   call render_sys_drawSpriteClipped

   renderNormal:
   ld b, e_h(ix)
   call render_sys_drawSprite
   ret

   render_item:
   ld b, e_h(ix)
   call render_sys_drawSpriteMask
   ret
    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;render_sys_player
;;Renderizar jugador
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: abcdhl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_sys_player:

   call entity_man_get_entity_array

   ld a, e_type(ix)
   ld b, #type_player
   and b
   cp b
   ret nz

   ld c, e_x(ix)
   ld b, e_y(ix)
   ld de, (back_buffer)
   call render_sys_getScreenPointer

   ld a, e_lastPtr(ix)
   ld e_lastPtr2(ix),a
   ld a, e_lastPtr+1(ix)   
   ld e_lastPtr2+1(ix),a
   
   ld d, e_sprite+1(ix)
   ld e, e_sprite(ix)
   ex de, hl ;Guardamos en 'hl' el sprite y en 'de' la posición
   ld c, e_w(ix)
   ld b, e_h(ix)
   ld e_lastPtr(ix), e
   ld e_lastPtr+1(ix), d

   call render_sys_drawSpriteMask

   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;render_sys_borrado_player
;;Renderizar jugador
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: hl: puntero de esa posicion
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: abcdhl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_sys_borrado_player:

   call entity_man_getPlayer

   ld d, e_lastPtr2+1(ix)
   ld e, e_lastPtr2(ix)
   ld c, e_w(ix)
   ld b, e_h(ix) 
   
   call game_man_getLastFila
   cp #0x07
   jr nc, azul2
   ld a, #0xC3
   jr borrar2

  azul2:
   ld a, #0x3C

  borrar2:
   call render_sys_drawSolidBox

   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;render_sys_getScreenPointer
;;Obtener puntero a zona de memoria
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: de: puntero inicial pantalla
;;      c: x
;;      b: y
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: hl: puntero de esa posicion
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: abcdhl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_sys_getScreenPointer:

   ld a,c
   cp #80
   jr c,render_sys_getScreenPointerseguir
   ld c,#0

  render_sys_getScreenPointerseguir:

   ;; Add up X coordinate to screen_memory, so that DE = (screen_start + x)
   ;; (DE += C, as DE = screen_memory, and C = x coordinate) 
   ld    a, e     ;; [1] / E = E + x   (As C = x)
   add   c        ;; [1] |   ( If E + x > 0xFF, Carry is set. We must Add it to D )
   ld    e, a     ;; [1] |
   adc   d        ;; [1] | D = D + (C + A) + Carry   (E = C + A)
   sub   e        ;; [1] | D = D + Carry
   ld    d, a     ;; [1] |   (If E + x did set carry, this will do D += 1, else D += 0)

   ;; Let y' = [ y / 8 ] = int(y / 8) (Integer division by 8)
   ;; Calculate 80y' = (64 + 16)y' (0x50 * y')
   ld    a, b     ;; [1] A = y coordinate (which is stored in b)
   and   #0xF8    ;; [2] A = 8y'   ( and 0xF8 is the same as 8 * int(y / 8) = 8y' )
   ld    l, a     ;; [1] | HL = 8y'  (H = 0, L = 8y')
   ld    h, #0    ;; [2] |
   ld    a, b     ;; [1] A = y coordinate (Save for later use, freeing BC)
   add  hl, hl    ;; [3] HL = 8y' + 8y' = 16y'
   add  hl, hl    ;; [3] HL = 16y' + 16y' = 32y'
   add  hl, hl    ;; [3] HL = 32y' + 32y' = 64y'
  
   ;; Calculate 2048 * (y % 8)  (0x800 * (y % 8))
   and   #0x07    ;; [2]  A = y % 8  (A had y coordinate)
   rlca           ;; [1]  A = 2 (y % 8)
   rlca           ;; [1]  A = 4 (y % 8)
   rlca           ;; [1]  A = 8 (y % 8)
   ;; Now [A 0x00] (Using a as HSB) would be 0x100 * A = 0x100 * 8 (y % 8 )
   ;; So, we only have to add A to H in order to have HL += 0x100 * 8 (y % 8)
   add   h        ;; [1]  | HL += 0x100 * 8 (y % 8)
   ld    h, a     ;; [1]  |

   ;; Add up everything and return
   add  hl, de    ;; [3] HL = 80 * [y / 8] + 2048 * (y % 8) + (screen_start + x)

   ret            ;; [3] Return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;render_sys_drawSprite
;;Obtener puntero a zona de memoria
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: de: puntero inicial pantalla
;;      hl: puntero de sprite
;;      de: puntero memoria video
;;       c: width
;;         b: height
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: hl: puntero de esa posicion
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: abcdhl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_sys_drawSprite:

   ld    a, #126           ;; [2] We need to jump 126 bytes (63 LDIs*2 bytes) minus the width of the sprite * 2 (2B)
   sub   c                 ;; [1]    to do as much LDIs as bytes the Sprite is wide
   sub   c                 ;; [1]
   ld (ds_drawSpriteWidth+#4), a ;; [4] Modify JR data to create the jump we need

   ld    a, b              ;; [1] A = Height (used as counter for the number of lines we have to copy)
   ex   de, hl             ;; [1] Instead of jumping over the next line, we do the inverse operation because 
                           ;; .... it is only 4 cycles and not 10, as a JP would be)

ds_drawSpriteWidth_next:
   ;; NEXT LINE
   ex   de, hl             ;; [1] HL and DE are exchanged every line to do 16bit maths with DE. 
                           ;; .... This line reverses it before proceeding to copy the next line.
ds_drawSpriteWidth:
   ;; Draw a sprite-line of n bytes
   ld   bc, #0x800  ;; [3] 0x800 bytes is the distance in memory from one pixel line to the next within every 8 pixel lines
                    ;; ... Each LDI performed will decrease this by 1, as we progress through memory copying the present line
   jr__0            ;; [3] Self modifying instruction: the '00' will be substituted by the required jump forward. 
                    ;; ... (Note: Writting JR 0 compiles but later it gives odd linking errors)
   ldi              ;; [5] <| 63 LDIs, which are able to copy up to 63 bytes each time.
   ldi              ;; [5]  | That means that each Sprite line should be 63 bytes width at most.
   ldi              ;; [5]  | The JR instruction at the start makes us ignore the LDIs we don't need 
   ldi              ;; [5] <| (jumping over them) That ensures we will be doing only as much LDIs 
   ldi              ;; [5] <| as bytes our sprite is wide.
   ldi              ;; [5]  |
   ldi              ;; [5]  |
   ldi              ;; [5] <|
   ldi              ;; [5] <|
   ldi              ;; [5]  |
   ldi              ;; [5]  |
   ldi              ;; [5] <|
   ldi              ;; [5] <|
   ldi              ;; [5]  |
   ldi              ;; [5]  |
   ldi              ;; [5] <|
   ldi              ;; [5] <|
   ldi              ;; [5]  |
   ldi              ;; [5]  |
   ldi              ;; [5] <|
   ldi              ;; [5]  |
   ldi              ;; [5] <|
   ldi              ;; [5] <|
   ldi              ;; [5]  |
   ldi              ;; [5]  |
   ldi              ;; [5] <|
   ldi              ;; [5] <|
   ldi              ;; [5]  |
   ldi              ;; [5]  |
   ldi              ;; [5] <|
   ldi              ;; [5]  |
   ldi              ;; [5] <|
   ldi              ;; [5] <|
   ldi              ;; [5]  |
   ldi              ;; [5]  |
   ldi              ;; [5] <|
   ldi              ;; [5] <|
   ldi              ;; [5]  |
   ldi              ;; [5]  |
   ldi              ;; [5] <|
   ldi              ;; [5]  |
   ldi              ;; [5] <|
   ldi              ;; [5] <|
   ldi              ;; [5]  |
   ldi              ;; [5]  |
   ldi              ;; [5] <|
   ldi              ;; [5] <|
   ldi              ;; [5]  |
   ldi              ;; [5]  |
   ldi              ;; [5] <|
   ldi              ;; [5]  |
   ldi              ;; [5] <|
   ldi              ;; [5] <|
   ldi              ;; [5]  |
   ldi              ;; [5]  |
   ldi              ;; [5] <|
   ldi              ;; [5] <|
   ldi              ;; [5]  |
   ldi              ;; [5]  |
   ldi              ;; [5] <|
   ldi              ;; [5] <|
   ldi              ;; [5]  |
   ldi              ;; [5]  |
 
   dec   a          ;; [1] Another line finished: we discount it from A
   ret   z          ;; [2/4] If that was the last line, we safely return

   ;; Jump destination pointer to the start of the next line in video memory
   ex   de, hl      ;; [1] DE has destination, but we have to exchange it with HL to be able to do 16bit maths
   add  hl, bc      ;; [3] We add 0x800 minus the width of the sprite (BC) to destination pointer 
   ld    b, a       ;; [1] Save A into B (B = A)
   ld    a, h       ;; [1] We check if we have crossed video memory boundaries (which will happen every 8 lines). 
                    ;; .... If that happens, bits 13,12 and 11 of destination pointer will be 0
   and   #0x38      ;; [2] leave out only bits 13,12 and 11 from new memory address (00xxx000 00000000)
   ld    a, b       ;; [1] Restore A from B (A = B)
   jp   nz, ds_drawSpriteWidth_next ;; [3] If any bit from {13,12,11} is not 0, we are still inside 
                                    ;; .... video memory boundaries, so proceed with next line

   ;; Every 8 lines, we cross the 16K video memory boundaries and have to
   ;; reposition destination pointer. That means our next line is 16K-0x50 bytes back
   ;; which is the same as advancing 48K+0x50 = 0xC050 bytes, as memory is 64K 
   ;; and our 16bit pointers cycle over it
   ld   bc, #0xC040           ;; [3] We advance destination pointer to next line
   add  hl, bc                ;; [3]  HL += 0xC050
   jp ds_drawSpriteWidth_next ;; [3] Continue copying

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;render_sys_drawSolidBox
;;Pintar cuadrado
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: 
;;      de: puntero memoria video
;;       a: color
;;       c: width
;;        b: height
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: abcdhl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_sys_drawSolidBox:


  push af
   ld a, d
   cp #0x80
   jr  nc, solidBox_next
   pop af
   ret

  ;; DrawSolidBox code starts here (immediately after getting parameters from stack)
  solidBox_next:
   pop af
   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; COPY2HL_n_INC Macro 
;;  Copy Register To (HL) and Increment HL
;; INPUT:
;;    reg8 - any 8-bits register
;;
.macro COPY2HL_n_INC reg8
   ld   (hl), reg8         ;; [2] (HL) = reg8
   inc   hl                ;; [2] HL++  
.endm

;; DrawSolidBox code starts here (immediately after getting parameters from stack)
   ld     h, d             ;; [1] / HL = DE  
   ld     l, e             ;; [1] \ HL Points to Video Memory, and DE holds a copy which won't be modified
   ld     e, a             ;; [1] E=A (Colour pattern)
   
   ;; Modify code using width to jump to ensure
   ;; that only width bytes are copied each line 
   ld     a, #128          ;; [2] We need to jump 128 bytes (64 COPY2HL_n_INC*2 bytes) minus the width of the sprite * 2 (2B)
   sub    c                ;; [1]  to do as much COPY2HL_n_INC as bytes the Sprite is wide
   sub    c                ;; [1]
   ld (_jr_offset), a      ;; [4] Modify JR data to create the jump we need
   
   ld     c, e             ;; [1] C=E (Colour pattern)
   ld     e, l             ;; [1] Restore DE = HL

_next_line:
_jr_offset = .+1
   jr__0                   ;; [3] Self modifying instruction: the '00' will be substituted by the required jump forward. 
                           ;; ... (Note: Writting JR 0 compiles but later it gives odd linking errors)  
;; 64 COPY2HL_n_INC c, which are able to copy up to 64 bytes each time.
;; That means that each Sprite line should be 64 bytes width at most.
;; The JR instruction at the start makes us ignore the COPY2HL_n_INC we don't need 
;; (jumping over them) That ensures we will be doing only as much COPY2HL_n_INC 
;; as bytes our sprite is wide.
.rept 64
   COPY2HL_n_INC c         ;; [4] (HL) = c, ++HL { Repeated 64 times }
.endm
 
   dec    b                ;; [1] Another line finished: we discount it from D
   ret    z                ;; [2/4] If that was the last line, we safely return

   ;; Jump destination pointer to the start of the next line in video memory
   ld     a, #0x8          ;; [2] / HL = DE = DE + 0x800
   add    d                ;; [1] | DE holds the pointer to the start of this line
   ld     h, a             ;; [1] | Adding 0x800 makes HL point to the start of
   ld     d, a             ;; [1] | the next line to be filled in with values
   ld     l, e             ;; [1] \ DE holds a copy which won't be modified
   ;; We check if we have crossed video memory boundaries (which will happen every 8 lines). 
   ;; ... If that happens, bits 13,12 and 11 of destination pointer will be 0
   and   #0x38             ;; [2] leave out only bits 13,12 and 11 from new memory address (00xxx000 00000000)
   jp    nz, _next_line    ;; [3] If any bit from {13,12,11} is not 0, we are still inside 
                           ;; ... video memory boundaries, so proceed with next line

   ;; Every 8 lines, we cross the 16K video memory boundaries and have to
   ;; reposition destination pointer. That means our next line is 16K-0x50 bytes back
   ;; which is the same as advancing 48K+0x50 = 0xC050 bytes, as memory is 64K 
   ;; and our 16bit pointers cycle over it
   ld    hl, #0xC040       ;; [3] We advance destination pointer to next line
   add   hl, de            ;; [3] HL = DE + 0xC050
   ld     d, h             ;; [1] / DE = HL
   ld     e, l             ;; [1] \
   jp   _next_line         ;; [3] Continue copying   

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;render_sys_limpiar
;;Borra toda la pantalla
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: dehl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_sys_limpiar:

   ld de, #0xC000
   ld a, #0xC3
   ld c, #64
   ld b, #256
   call render_sys_drawSolidBox

   ld de, #0x8000
   ld a, #0xC3
   ld c, #64
   ld b, #256
   call render_sys_drawSolidBox

   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;render_sys_fondo
;;Dibuja fondo
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:abcde
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_sys_fondo:

   ld hl,(back_buffer) 
   ld bc, #0x0040
   add hl,bc
   ex de,hl
   ld a,  #0x3C
   ld c, #64
   ld b, #120
   call render_sys_drawSolidBox

   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;render_sys_completarMar
;;Dibuja fondo
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:abcde
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_sys_completarMar:

   ld hl,(back_buffer) 
   ld bc, #0x23C0
   add hl,bc

   ex de, hl
   ld a,  #0x3C
   ld c, #64
   ld b, #4
   call render_sys_drawSolidBox

   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;render_sys_inicializarTiempo
;;Dibuja barra de tiempo
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:hlcbdeiy
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_sys_inicializarTiempo:

    ;Dibujamos la string TIME
    ld    h, #9 ;D = Background PEN (0)
    ld    l, #8 ;E = Foreground PEN (3)
    call cpct_setDrawCharM0_asm  
    ld c, #0x30 ;Coordenada X
    ld b, #0xF8 ;Coordenada Y
    ld de, (back_buffer)
    call render_sys_getScreenPointer     
    ld   iy, #string_time ;Puntero a la string
    call myDrawStringM0  ;Dibuja la string

    ;PALETTE = 26 0 1 2 5 9 11 12 13 15 17 20 22 23 24 25
    ;Barra verde
    ld  hl, (back_buffer)
    ld  de, #0x07C2
    add hl, de
    ex  de, hl
    ld  a, #0x33
    ld  c, #45
    ld  b, #7
    call render_sys_drawSolidBox

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;render_update_tiempo
;;Va reduciendo la barra de tiempo
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:abcde
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_update_tiempo:  

    call game_man_getTime
    ld hl, #0xC000
    ld de, #0x07C2
    add hl,de
    ex de,hl
    ld a,  #0xC3
    ld b, #7
    push bc
    call render_sys_drawSolidBox

    ld hl, #0x8000
    ld de, #0x07C2
    add hl,de
    ex de,hl
    ld a,  #0xC3
    pop bc
    call render_sys_drawSolidBox

    ret
    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;render_sys_textoNivel
;;Texto nivel
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: a nivel
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:ab
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_sys_textoNivel:

    push af
    ;Dibujamos la string TIME
    ld    h, #9 ;D = Background PEN (0)
    ld    l, #8 ;E = Foreground PEN (3)
    call cpct_setDrawCharM0_asm  
    ld c, #0x00 ;Coordenada X
    ld b, #0x00 ;Coordenada Y
    ld de, (back_buffer) 
    call render_sys_getScreenPointer     
    ld   iy, #string_nivel ;Puntero a la string
    pop af
    push bc
    ld b,#0

    textoNivelloop:
    cp #0x0A
    jr c,textoNivelMenor
    sub #0x0A
    inc b
    jr textoNivelloop

    textoNivelMenor:

    push af
    ld a,b
    cp #0
    jr z,textoNivelacabar

    pop af

    add a, #48
    ld  7(iy),a

    ld a,b
    add a,#48
    ld  6(iy),a
    jr textoNivelacabar2

    textoNivelacabar:
    pop af
    add a, #48
    ld  6(iy),a

    textoNivelacabar2:
    pop bc

    call myDrawStringM0  ;Dibuja la string

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;render_sys_resetTextoNivel
;;Texto nivel
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: a nivel
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register:ab
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_sys_resetTextoNivel:
 
    ld   iy, #string_nivel ;Puntero a la string

    ld a, #0x20
    ld  7(iy),a

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;render_sys_drawHud
;;Dibuja hud 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: a: x
;;       hl: puntero a sprite
;;       de: primera posicion del buffer
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: cbdehl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_sys_drawHud:

  push hl
  ld c, a
  ld b, #0x00
  call render_sys_getScreenPointer
  ex de, hl
  pop hl

  ld c, #0x4
  ld b, #0x8
  call render_sys_drawSprite

  ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;render_sys_drawSpriteMask
;;Renderizar sprite con mascara de blanco
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Input Parameters (6 bytes):
;;  (2B HL) sprite - Source Sprite Pointer (array with pixel and mask data)
;;  (2B DE) memory - Destination video memory pointer
;;  (1B C ) width  - Sprite Width in *bytes* (>0) (Beware, *not* in pixels!)
;;  (1B B ) height - Sprite Height in bytes (>0)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: hl: puntero de esa posicion
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: abcdhl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_sys_drawSpriteMask:

   push ix         ;; [5] Save IX regiter before using it as temporal var
   ld__ixl_c       ;; [2] Save Sprite Width into IXL for later use

dms_sprite_height_loop:
   push de         ;; [4] Save DE for later use (jump to next screen line)

dms_sprite_width_loop:
   ld    a ,(hl)   ;; sprite
   jr z, dms_sprite_width_fondo

   push bc
   push de
   and #0b10101010
   cp #0
   jr nz, dms_sprite_width_1no
     ld a,(de)   ;fondo
     and #0b10101010
    
   dms_sprite_width_1no:
 
   ld c,a

   ld  a ,(hl)   ;; sprite
   and #0b01010101
   cp #0
   jr nz, dms_sprite_width_2no
     ld a,(de)   ;fondo
     and #0b01010101
   dms_sprite_width_2no:

   add c
   ld  (de), a     ;; [2] Save modified background + sprite data information into memory

   pop de
   pop bc

   dms_sprite_width_fondo:
   inc  de         ;; [2] Next bytes (sprite and memory)
   inc  hl         ;; [2] 

   dec   c         ;; [1] C holds sprite width, we decrease it to count pixels in this line.
   jr   nz,dms_sprite_width_loop;; [2/3] While not 0, we are still painting this sprite line 
                                ;;      - When 0, we have to jump to next pixel line

   pop  de         ;; [3] Recover DE from stack. We use it to calculate start of next pixel line on screen

   dec   b         ;; [1] B holds sprite height. We decrease it to count another pixel line finished
   jr    z,dms_sprite_copy_ended;; [2/3] If 0, we have finished the last sprite line.
                                ;;      - If not 0, we have to move pointers to the next pixel line

   ld__c_ixl       ;; [2] Restore Sprite Width into C

   ld    a, d      ;; [1] Start of next pixel line normally is 0x0800 bytes away.
   add   #0x08     ;; [2]    so we add it to DE (just by adding 0x08 to D)
   ld    d, a      ;; [1]
   and   #0x38     ;; [2] We check if we have crossed memory boundary (every 8 pixel lines)
   jr   nz, dms_sprite_height_loop ;; [2/3]  by checking the 4 bits that identify present memory line. 
                                   ;; ....  If 0, we have crossed boundaries

dms_sprite_8bit_boundary_crossed:
   ld    a, e      ;; [1] DE = DE + 0xC050h
   add   #0x40     ;; [2] -- Relocate DE pointer to the start of the next pixel line:
   ld    e, a      ;; [1] -- DE is moved forward 3 memory banks plus 50 bytes (4000h * 3) 
   ld    a, d      ;; [1] -- which effectively is the same as moving it 1 bank backwards and then
   adc   #0xC0     ;; [2] -- 50 bytes forwards (which is what we want to move it to the next pixel line)
   ld    d, a      ;; [1] -- Calculations are made with 8 bit maths as it is faster than other alternatives here

   jr  dms_sprite_height_loop ;; [3] Jump to continue with next pixel line

dms_sprite_copy_ended:
   pop  ix         ;; [5] Restore IX before returning
   ret             ;; [3] Return to caller

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;render_sys_drawSpriteClipped
;;Renderizar clipping derecha
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input:
;;       a: width sprite que no se renderiza
;;      hl: puntero de sprite
;;      de: puntero memoria video
;;       c: width a dibujar
;;       b: height
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: hl: puntero de esa posicion
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: abcdhl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_sys_drawSpriteClipped:

   push ix         ;; [5] Save IX regiter before using it as temporal var
   ld__ixl_c       ;; [2] Save Sprite Width into IXL for later use


   cp #80
   jr c,dms_sprite_clipped_height_loopotro
   push bc
   neg
   ld b,#0
   ld c, a
   add hl,bc

   pop bc
   dms_sprite_clipped_height_loopotro:
   ld__ixh_a

dms_sprite_clipped_height_loop:
   push de         ;; [4] Save DE for later use (jump to next screen line)

dms_sprite_clipped_width_loop:

   ld    a ,(hl)   ;; sprite
   ld  (de), a     ;; [2] Save modified background + sprite data information into memory


   dms_sprite_clipped_width_fondo:
   inc  de         ;; [2] Next bytes (sprite and memory)
   inc  hl         ;; [2] 

   dec   c         ;; [1] C holds sprite width, we decrease it to count pixels in this line.
   jr   nz,dms_sprite_clipped_width_loop;; [2/3] While not 0, we are still painting this sprite line 
                                ;;      - When 0, we have to jump to next pixel line

   pop  de         ;; [3] Recover DE from stack. We use it to calculate start of next pixel line on screen

   dec   b         ;; [1] B holds sprite height. We decrease it to count another pixel line finished
   jr    z,dms_sprite_clipped_copy_ended;; [2/3] If 0, we have finished the last sprite line.
                                ;;      - If not 0, we have to move pointers to the next pixel line
   ld__a_ixh
   push bc
   ld b,#0
   ld c, a
   add hl,bc
   pop bc


   ld__c_ixl       ;; [2] Restore Sprite Width into C

   ld    a, d      ;; [1] Start of next pixel line normally is 0x0800 bytes away.
   add   #0x08     ;; [2]    so we add it to DE (just by adding 0x08 to D)
   ld    d, a      ;; [1]
   and   #0x38     ;; [2] We check if we have crossed memory boundary (every 8 pixel lines)
   jr   nz, dms_sprite_clipped_height_loop ;; [2/3]  by checking the 4 bits that identify present memory line. 
                                   ;; ....  If 0, we have crossed boundaries

dms_sprite_clipped_8bit_boundary_crossed:
   ld    a, e      ;; [1] DE = DE + 0xC050h
   add   #0x40     ;; [2] -- Relocate DE pointer to the start of the next pixel line:
   ld    e, a      ;; [1] -- DE is moved forward 3 memory banks plus 50 bytes (4000h * 3) 
   ld    a, d      ;; [1] -- which effectively is the same as moving it 1 bank backwards and then
   adc   #0xC0     ;; [2] -- 50 bytes forwards (which is what we want to move it to the next pixel line)
   ld    d, a      ;; [1] -- Calculations are made with 8 bit maths as it is faster than other alternatives here

   jr  dms_sprite_clipped_height_loop ;; [3] Jump to continue with next pixel line

dms_sprite_clipped_copy_ended:
   pop  ix         ;; [5] Restore IX before returning
   ret             ;; [3] Return to caller

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;render_sys_borrarMask
;;Renderizar sprite con mascara de blanco
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Input Parameters (6 bytes):
;;  (1B  A) color
;;  (2B HL) sprite - Source Sprite Pointer (array with pixel and mask data)
;;  (2B DE) memory - Destination video memory pointer
;;  (1B C ) width  - Sprite Width in *bytes* (>0) (Beware, *not* in pixels!)
;;  (1B B ) height - Sprite Height in bytes (>0)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: hl: puntero de esa posicion
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: abcdhl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_sys_borrarMask:

   push ix         ;; [5] Save IX regiter before using it as temporal var
   ld__ixl_c       ;; [2] Save Sprite Width into IXL for later use
   ld__ixh_a

dms_sprite_borrar_height_loop:
   push de         ;; [4] Save DE for later use (jump to next screen line)

dms_sprite_borrar_width_loop:
   ld    a ,(hl)   ;; sprite
   jr z, dms_sprite_borrar_width_fondo

   push bc
   push de
   and #0b10101010
   cp #0
   jr nz, dms_sprite_borrar_width_1no
     ld a,(de)   ;fondo
     and #0b10101010
    jr dms_sprite_borrar_width_1noseguir

   dms_sprite_borrar_width_1no:
    ld__a_ixh
    and #0b10101010

   dms_sprite_borrar_width_1noseguir:

   ld c,a

   ld  a ,(hl)   ;; sprite
   and #0b01010101
   cp #0
   jr nz, dms_sprite_borrar_width_2no
     ld a,(de)   ;fondo
     and #0b01010101
    jr dms_sprite_borrar_width_2noseguir
   dms_sprite_borrar_width_2no:
    ld__a_ixh
    and #0b01010101
   dms_sprite_borrar_width_2noseguir:
   add c
   ld  (de), a     ;; [2] Save modified background + sprite data information into memory

   pop de
   pop bc

   dms_sprite_borrar_width_fondo:
   inc  de         ;; [2] Next bytes (sprite and memory)
   inc  hl         ;; [2] 

   dec   c         ;; [1] C holds sprite width, we decrease it to count pixels in this line.
   jr   nz,dms_sprite_borrar_width_loop;; [2/3] While not 0, we are still painting this sprite line 
                                ;;      - When 0, we have to jump to next pixel line

   pop  de         ;; [3] Recover DE from stack. We use it to calculate start of next pixel line on screen

   dec   b         ;; [1] B holds sprite height. We decrease it to count another pixel line finished
   jr    z,dms_sprite_borrar_copy_ended;; [2/3] If 0, we have finished the last sprite line.
                                ;;      - If not 0, we have to move pointers to the next pixel line

   ld__c_ixl       ;; [2] Restore Sprite Width into C

   ld    a, d      ;; [1] Start of next pixel line normally is 0x0800 bytes away.
   add   #0x08     ;; [2]    so we add it to DE (just by adding 0x08 to D)
   ld    d, a      ;; [1]
   and   #0x38     ;; [2] We check if we have crossed memory boundary (every 8 pixel lines)
   jr   nz, dms_sprite_borrar_height_loop ;; [2/3]  by checking the 4 bits that identify present memory line. 
                                   ;; ....  If 0, we have crossed boundaries

dms_sprite_borrar_8bit_boundary_crossed:
   ld    a, e      ;; [1] DE = DE + 0xC050h
   add   #0x40     ;; [2] -- Relocate DE pointer to the start of the next pixel line:
   ld    e, a      ;; [1] -- DE is moved forward 3 memory banks plus 50 bytes (4000h * 3) 
   ld    a, d      ;; [1] -- which effectively is the same as moving it 1 bank backwards and then
   adc   #0xC0     ;; [2] -- 50 bytes forwards (which is what we want to move it to the next pixel line)
   ld    d, a      ;; [1] -- Calculations are made with 8 bit maths as it is faster than other alternatives here

   jr  dms_sprite_borrar_height_loop ;; [3] Jump to continue with next pixel line

dms_sprite_borrar_copy_ended:
   pop  ix         ;; [5] Restore IX before returning

   ret             ;; [3] Return to caller

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;render_sys_borrarMask
;;Renderizar sprite con mascara de blanco
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Input Parameters (6 bytes):
;;  (2B HL) video_memory - Video memory location where the character will be drawn
;;  (1B A ) ascii        - Character to be drawn (ASCII code)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: hl: puntero de esa posicion
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: abcdhl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_sys_drawColorString:

  ex de,hl
  ld hl,#font_array
  sub #32

  cp #0
  jr z,render_sys_drawColorStringFinal
  
  cp #33
  jr c,render_sys_drawColorStringNumeros
  sub #22
  jr render_sys_drawColorStringFinal

  render_sys_drawColorStringNumeros:
  sub #15

  render_sys_drawColorStringFinal:
  add a,a
  ld b,#0
  ld c,a
  add hl,bc
  push hl
  pop ix
  ld h,1(ix)
  ld l,(ix)
 
;;  (2B HL) sprite - Source Sprite Pointer (array with pixel and mask data)
;;  (2B DE) memory - Destination video memory pointer
;;  (1B C ) width  - Sprite Width in *bytes* (>0) (Beware, *not* in pixels!)
;;  (1B B ) height - Sprite Height in bytes (>0)

  ld b,#Height8
  ld c,#Width4
  call render_sys_drawSpriteMask

  ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;render_sys_change_screen
;;Renderizar sprite con mascara de blanco
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Input Parameters (6 bytes):
;;  (2B HL) video_memory - Video memory location where the character will be drawn
;;  (1B A ) ascii        - Character to be drawn (ASCII code)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: hl: puntero de esa posicion
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: abcdhl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_sys_change_screen:
  f_change_screen = . + 1
  jp change_screen_to_8000


change_screen_to_8000:
  ld  de, #0x0C20
  call render_sys_set_crtc

  ld hl, #0xC000
  ld (back_buffer), hl

  ld hl, #change_screen_to_C000
  ld (f_change_screen), hl

  ret

change_screen_to_C000:
  ld  de, #0x0C30
  call render_sys_set_crtc

  ld hl, #0x8000
  ld (back_buffer), hl

  ld hl, #change_screen_to_8000
  ld (f_change_screen), hl

  ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;render_sys_getBufferPtr
;;Renderizar sprite con mascara de blanco
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Output: de: puntero de buffer de video
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: abcdhl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_sys_getBufferPtr:

  ld de, (back_buffer)

  ret

;render_sys_setBufferPtr
;;Set de la posicion del buffer de video
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: de: puntero de buffer de video
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: abcdhl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_sys_setBufferPtr:

  ld (back_buffer),de

  ret

;render_sys_getBufferScreenPtr
;;get de la posicion del buffer de video
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: de: puntero de buffer de video
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: abcdhl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_sys_getBufferScreenPtr:

  ld a,(back_buffer+1)
  cp #0x80
  jr z, ponerC0

  ld de,#0x8000
  ret

  ponerC0:
  ld de,#0xC000
  ret

;;render_sys_dibujar_transicion
;;get de la posicion del buffer de video
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Input: de: puntero de buffer de video
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Modified register: abcdhl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_sys_dibujar_transicion:

  call entity_man_getPlayer
  ld a, e_fila(ix)
  cp #9
  ret nc
  cp #5
  ret c

  ld hl,(back_buffer) 
  ld bc, #0x13C0
  add hl,bc

  ex de, hl
  ld a,  #0x3C
  ld c, #64
  ld b, #6
  call render_sys_drawSolidBox

  ld hl,(back_buffer) 
  ld bc, #0x0400
  add hl,bc

  ex de, hl
  ld a,  #0xC3
  ld c, #64
  ld b, #6
  call render_sys_drawSolidBox

  ret